"""
玩家底力分析
"""

from dataclasses import dataclass

from ..score import PlayerMaiScore
from .song_tags import get_songs_tags


@dataclass
class PlayerStrength:
    patterns_strengths: dict[str, int]
    difficulty_strengths: dict[str, int]
    song_evaluates: dict[str, int]


def _extract_values(source: dict, *fields: str) -> dict:
    output = {}

    for field in fields:
        output[field] = source.get(field, 0)

    return output


def get_player_strength(scores: list[PlayerMaiScore]) -> PlayerStrength:
    player_strengths: dict[str, int] = {}

    for score in scores:
        song_type = "dx" if score.song_type.value == "dx" else "std"
        song_diff = score.song_difficulty.name.lower()
        if song_diff not in ["remaster", "master", "expert"]:
            continue

        tags = get_songs_tags(score.song_name, song_type, song_diff)  # type: ignore

        for tag in tags:
            player_strengths[tag] = player_strengths.get(tag, 0) + 1

    patterns_strengths = _extract_values(
        player_strengths,
        "转圈",
        "绝赞段",
        "扫键",
        "散打",
        "交互",
        "反手",
        "一笔画",
        "错位",
        "跳拍",
        "纵连",
        "爆发",
        "拆弹",
        "定拍",
        "大位移",
    )
    difficulty_strengths = _extract_values(player_strengths, "水", "诈称谱")
    difficulty_strengths["正常铺"] = len(scores) - sum(difficulty_strengths.values())
    song_evaluates = _extract_values(player_strengths, "底力谱", "体力谱", "高物量", "键盘谱", "星星谱")

    return PlayerStrength(patterns_strengths, difficulty_strengths, song_evaluates)



"""
绘图配置文件
包含路径配置、字体配置、颜色配置以及各种映射表
"""

from pathlib import Path
from typing import Dict

from ..config import config

# Configuration
STATIC_DIR = Path(config.static_resource_path)
MAI_DIR = STATIC_DIR / "mai"
PIC_DIR = MAI_DIR / "pic"
COVER_DIR = MAI_DIR / "cover"
PLATE_DIR = MAI_DIR / "plate"
ICON_DIR = MAI_DIR / "icon"

# Fonts
# Adjust these paths if necessary based on your actual font files
FONT_MAIN = (
    STATIC_DIR / "ResourceHanRoundedCN-Bold.ttf" if not config.scorelist_font_main else Path(config.scorelist_font_main)
)
FONT_NUM = STATIC_DIR / "Torus SemiBold.otf" if not config.scorelist_font_num else Path(config.scorelist_font_num)

# Colors
TEXT_COLOR = (124, 129, 255, 255)

# Mappings
SCORE_RANK_L: Dict[str, str] = {
    "d": "D",
    "c": "C",
    "b": "B",
    "bb": "BB",
    "bbb": "BBB",
    "a": "A",
    "aa": "AA",
    "aaa": "AAA",
    "s": "S",
    "sp": "Sp",
    "ss": "SS",
    "ssp": "SSp",
    "sss": "SSS",
    "sssp": "SSSp",
}
FCL: Dict[str, str] = {"fc": "FC", "fcp": "FCp", "ap": "AP", "app": "APp"}
FSL: Dict[str, str] = {"fs": "FS", "fsp": "FSp", "fsd": "FSD", "fsdp": "FSDp", "sync": "Sync"}
GENRE_MAPPING: Dict[str, str] = {
    "POPSアニメ": "anime",
    "niconicoボーカロイド": "niconico",
    "東方Project": "touhou",
    "ゲームバラエティ": "game",
    "maimai": "maimai",
    "オンゲキCHUNITHM": "ongeki",
}



"""add maimaipy_identifier

迁移 ID: 5c791c303fe1
父迁移: de4344fd6e4c
创建时间: 2026-01-02 22:43:32.110967

"""

from __future__ import annotations

from collections.abc import Sequence

import sqlalchemy as sa
from alembic import op

revision: str = "5c791c303fe1"
down_revision: str | Sequence[str] | None = "de4344fd6e4c"
branch_labels: str | Sequence[str] | None = None
depends_on: str | Sequence[str] | None = None


def upgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("nonebot_plugin_rikka_userbindinfo", schema=None) as batch_op:
        batch_op.add_column(sa.Column("maimaipy_identifier", sa.String(), nullable=True))

    # ### end Alembic commands ###


def downgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("nonebot_plugin_rikka_userbindinfo", schema=None) as batch_op:
        batch_op.drop_column("maimaipy_identifier")

    # ### end Alembic commands ###



from pathlib import Path
from typing import List, Optional

from PIL import Image, ImageDraw

from ..config import config
from ..database.crud import MaiSongORM
from ..score import PlayerMaiInfo, PlayerMaiScore
from ._config import (
    COVER_DIR,
    FCL,
    FONT_MAIN,
    FONT_NUM,
    FSL,
    ICON_DIR,
    PIC_DIR,
    PLATE_DIR,
    SCORE_RANK_L,
)
from .utils import DrawText, change_column_width, coloum_width


class ScoreBaseImage:
    """
    成绩绘图基类，提供基础的绘图资源和方法
    """

    text_color = (124, 129, 255, 255)
    t_color = [
        (255, 255, 255, 255),
        (255, 255, 255, 255),
        (255, 255, 255, 255),
        (255, 255, 255, 255),
        (138, 0, 226, 255),
    ]
    id_color = [(129, 217, 85, 255), (245, 189, 21, 255), (255, 129, 141, 255), (159, 81, 220, 255), (138, 0, 226, 255)]
    bg_color = [
        (111, 212, 61, 255),
        (248, 183, 9, 255),
        (255, 129, 141, 255),
        (159, 81, 220, 255),
        (219, 170, 255, 255),
    ]

    def __init__(self, image: Optional[Image.Image] = None) -> None:
        """
        初始化 ScoreBaseImage

        :param image: 可选的 PIL Image 对象，如果提供则在其上绘图
        """
        self.bg = image
        self._image_cache: dict[tuple[str, Optional[tuple[int, int]], bool, bool], Image.Image] = {}
        self._diff_paths = [
            PIC_DIR / "b50_score_basic.png",
            PIC_DIR / "b50_score_advanced.png",
            PIC_DIR / "b50_score_expert.png",
            PIC_DIR / "b50_score_master.png",
            PIC_DIR / "b50_score_remaster.png",
        ]
        self.id_diff = [Image.new("RGBA", (55, 10), color) for color in self.bg_color]

    def _get_image(
        self,
        path: Path,
        size: Optional[tuple[int, int]] = None,
        with_opacity: bool = False,
        convert_rgba: bool = False,
    ) -> Image.Image:
        key = (str(path), size, with_opacity, convert_rgba)
        cached = self._image_cache.get(key)
        if cached is not None:
            return cached

        img = Image.open(path)
        if convert_rgba:
            img = img.convert("RGBA")
        if size is not None:
            img = img.resize(size)
        if with_opacity:
            img = self._with_opacity(img)

        self._image_cache[key] = img
        return img

    @property
    def title_lengthen_bg(self) -> Image.Image:
        return self._get_image(PIC_DIR / "title-lengthen.png")

    @property
    def design_bg(self) -> Image.Image:
        return self._get_image(PIC_DIR / "design.png")

    def _get_diff_bg(self, index: int) -> Image.Image:
        return self._get_image(self._diff_paths[index], with_opacity=True)

    def reset_im(self):
        """重置当前图像内容为纯背景图"""
        bg_path = config.scorelist_bg or PIC_DIR / "b50_bg.png"
        self._im = self.bg or Image.open(bg_path).convert("RGBA").resize((1400, 1600))
        dr = ImageDraw.Draw(self._im)
        self._sy = DrawText(dr, FONT_MAIN)
        self._tb = DrawText(dr, FONT_NUM)

    @staticmethod
    def _find_ra_pic(rating: int) -> str:
        """
        根据 Rating 获取对应的 Rating 图片文件名
        """
        if rating < 1000:
            num = "01"
        elif rating < 2000:
            num = "02"
        elif rating < 4000:
            num = "03"
        elif rating < 7000:
            num = "04"
        elif rating < 10000:
            num = "05"
        elif rating < 12000:
            num = "06"
        elif rating < 13000:
            num = "07"
        elif rating < 14000:
            num = "08"
        elif rating < 14500:
            num = "09"
        elif rating < 15000:
            num = "10"
        else:
            num = "11"
        return f"UI_CMN_DXRating_{num}.png"

    def _find_match_level(self, course_rank: int) -> str:
        """
        获取段位图片文件名
        """
        if course_rank <= 10:
            num = f"{course_rank:02d}"
        else:
            num = f"{course_rank + 1:02d}"
        return f"UI_DNM_DaniPlate_{num}.png"

    def draw_profile(self, player_info: PlayerMaiInfo, all_clear_rank: Optional[Image.Image] = None):
        """
        绘制 Profile 部分

        :param player_info: 玩家信息对象
        :type player_info: PlayerMaiInfo
        :param all_clear_rank: 全部达成的成绩等级
        """
        logo = self._with_opacity(
            Image.open(PIC_DIR / "logo.png").resize((249, 120)),
        )
        dx_rating = self._with_opacity(
            Image.open(PIC_DIR / self._find_ra_pic(player_info.rating)).resize((186, 35)),
        )
        name_img = self._with_opacity(
            Image.open(PIC_DIR / "Name.png"),
        )
        match_level = self._with_opacity(
            Image.open(PIC_DIR / self._find_match_level(player_info.course_rank)).resize((80, 32)),
        )
        class_level = self._with_opacity(
            Image.open(PIC_DIR / f"UI_FBR_Class_{player_info.class_rank:02d}.png").resize((90, 54)),
        )

        self._im.alpha_composite(logo, (14, 60))

        # Plate
        if player_info.name_plate:
            plate_path = PLATE_DIR / f"{player_info.name_plate.id}.png"
            if plate_path.exists():
                plate = Image.open(plate_path).resize((800, 130))
            else:
                plate = Image.open(PIC_DIR / "UI_Plate_300501.png").resize((800, 130))
        else:
            plate = Image.open(PIC_DIR / "UI_Plate_300501.png").resize((800, 130))
        plate = self._with_opacity(plate)
        self._im.alpha_composite(plate, (300, 60))

        # Icon
        if player_info.icon:
            icon_path = ICON_DIR / f"{player_info.icon.id}.png"
            if icon_path.exists():
                icon = Image.open(icon_path).resize((120, 120))
            else:
                icon = Image.open(PIC_DIR / "UI_Icon_309503.png").resize((120, 120))
        else:
            icon = Image.open(PIC_DIR / "UI_Icon_309503.png").resize((120, 120))
        icon = self._with_opacity(icon)
        self._im.alpha_composite(icon, (305, 65))

        self._im.alpha_composite(dx_rating, (435, 72))
        rating_str = f"{player_info.rating:05d}"
        for n, i in enumerate(rating_str):
            digit = self._with_opacity(
                Image.open(PIC_DIR / f"UI_NUM_Drating_{i}.png").resize((17, 20)),
            )
            self._im.alpha_composite(digit, (520 + 15 * n, 80))
        self._im.alpha_composite(name_img, (435, 115))
        self._im.alpha_composite(match_level, (625, 120))
        self._im.alpha_composite(class_level, (620, 60))

        self._sy.draw(445, 135, 25, player_info.name, (0, 0, 0, 255), "lm")

        # All Clear Rank
        if all_clear_rank:
            # all_clear_rank = all_clear_rank.resize(())
            self._im.alpha_composite(
                self._with_opacity(all_clear_rank),
                (1125, 50),
            )

    @staticmethod
    def _with_opacity(img: Image.Image, opacity: float = config.scorelist_element_opacity) -> Image.Image:
        """Return a copy of image with its alpha multiplied by opacity (0.0~1.0)."""

        if opacity is None:
            opacity = 1.0
        if opacity >= 0.999:
            return img.convert("RGBA") if img.mode != "RGBA" else img

        opacity = max(0.0, min(1.0, float(opacity)))
        rgba = img.convert("RGBA")
        if opacity <= 0.0:
            out = rgba.copy()
            out.putalpha(0)
            return out

        r, g, b, a = rgba.split()
        a = a.point(lambda p: int(p * opacity))
        out = rgba.copy()
        out.putalpha(a)
        return out

    def draw_footer(self):
        self._sy.draw(
            700,
            1570,
            27,
            "Designed by Yuri-YuzuChaN & BlueDeer233 & TomeChen. Generated by Nonebot-Plugin-Rikka",
            self.text_color,
            "mm",
            5,
            (255, 255, 255, 255),
        )

    def whiledraw(self, data: List[PlayerMaiScore], height: int = 235) -> None:
        """
        循环绘制成绩列表

        :param data: 成绩列表
        :param height: 自定义起始高度 (b35: 235; b15: 1085)
        """
        dy = 114
        y = height

        for num, info in enumerate(data):
            if num % 5 == 0:
                x = 16
                y += dy if num != 0 else 0
            else:
                x += 276

            cover_path = COVER_DIR / f"{info.song_id}.png"
            if not cover_path.exists():
                cover_path = COVER_DIR / f"{info.song_id + 10000}.png"

            if not cover_path.exists():
                # Fallback or handle missing cover
                cover = Image.new("RGBA", (75, 75), (0, 0, 0, 0))
            else:
                cover = Image.open(cover_path).resize((75, 75))

            version_path = PIC_DIR / f'{"DX" if info.song_type.value == "dx" else "SD"}.png'
            version = Image.open(version_path).resize((37, 14))

            rate_str = SCORE_RANK_L.get(info.rate.value, "D")
            rate = Image.open(PIC_DIR / f"UI_TTR_Rank_{rate_str}.png").resize((63, 28))

            max_dx_score: int | None = None
            song_obj = MaiSongORM.get_song_sync(info.song_id)
            if song_obj:
                diff_value = info.song_difficulty.value
                diff = (
                    song_obj.difficulties.dx[diff_value]
                    if info.song_type.value == "dx"
                    else song_obj.difficulties.standard[diff_value]
                )
                max_dx_score = diff.notes.total * 3

            self._im.alpha_composite(self._get_diff_bg(info.song_difficulty.value), (x, y))
            self._im.alpha_composite(cover, (x + 12, y + 12))
            self._im.alpha_composite(version, (x + 51, y + 91))
            self._im.alpha_composite(rate, (x + 92, y + 78))

            if info.fc:
                fc_str = FCL.get(info.fc.value, "")
                if fc_str:
                    fc = Image.open(PIC_DIR / f"UI_MSS_MBase_Icon_{fc_str}.png").resize((34, 34))
                    self._im.alpha_composite(fc, (x + 154, y + 77))
            if info.fs:
                fs_str = FSL.get(info.fs.value, "")
                if fs_str:
                    fs = Image.open(PIC_DIR / f"UI_MSS_MBase_Icon_{fs_str}.png").resize((34, 34))
                    self._im.alpha_composite(fs, (x + 185, y + 77))

            if info.dx_star:
                # dx_star is int (1-5)
                if info.dx_star > 0:
                    self._im.alpha_composite(
                        Image.open(PIC_DIR / f"UI_GAM_Gauge_DXScoreIcon_0{info.dx_star}.png").resize((47, 26)),
                        (x + 217, y + 80),
                    )

            self._tb.draw(x + 26, y + 98, 13, info.song_id, self.id_color[info.song_difficulty.value], anchor="mm")
            # Song title
            title = info.song_name
            if coloum_width(title) > 12:
                title = change_column_width(title, 12) + "..."
            self._sy.draw(x + 93, y + 14, 14, title, self.t_color[info.song_difficulty.value], anchor="lm")
            # Play Count
            if info.play_count:
                self._sy.draw(
                    x + 215, y + 14, 14, f"PC: {info.play_count}", self.t_color[info.song_difficulty.value], anchor="lm"
                )
            # Achievements
            self._tb.draw(
                x + 93, y + 38, 30, f"{info.achievements:.4f}%", self.t_color[info.song_difficulty.value], anchor="lm"
            )
            if max_dx_score is not None:
                self._tb.draw(
                    x + 209,
                    y + 65,
                    15,
                    f"{info.dx_score}/{max_dx_score}",
                    self.t_color[info.song_difficulty.value],
                    anchor="mm",
                )
            song_level_string = "{:.1f}".format(info.song_level_value) if info.song_level_value else info.song_level
            self._tb.draw(
                x + 93,
                y + 65,
                15,
                f"{song_level_string} -> {round(info.dx_rating)}",
                self.t_color[info.song_difficulty.value],
                anchor="lm",
            )



from io import BytesIO
from pathlib import Path
from typing import Optional, Tuple, Union

from PIL import Image, ImageDraw, ImageFont, ImageOps

from ..score import PlayerMaiScore
from ._config import PIC_DIR


def get_char_width(o: int) -> int:
    """
    获取字符宽度

    :param o: 字符的 Unicode 编码
    :return: 字符宽度 (0, 1, 2)
    """
    widths = [
        (126, 1),
        (159, 0),
        (687, 1),
        (710, 0),
        (711, 1),
        (727, 0),
        (733, 1),
        (879, 0),
        (1154, 1),
        (1161, 0),
        (4347, 1),
        (4447, 2),
        (7467, 1),
        (7521, 0),
        (8369, 1),
        (8426, 0),
        (9000, 1),
        (9002, 2),
        (11021, 1),
        (12350, 2),
        (12351, 1),
        (12438, 2),
        (12442, 0),
        (19893, 2),
        (19967, 1),
        (55203, 2),
        (63743, 1),
        (64106, 2),
        (65039, 1),
        (65059, 0),
        (65131, 2),
        (65279, 1),
        (65376, 2),
        (65500, 1),
        (65510, 2),
        (120831, 1),
        (262141, 2),
        (1114109, 1),
    ]
    if o == 0xE or o == 0xF:
        return 0
    for num, wid in widths:
        if o <= num:
            return wid
    return 1


def coloum_width(s: str) -> int:
    """
    计算字符串的总宽度

    :param s: 输入字符串
    :return: 总宽度
    """
    res = 0
    for ch in s:
        res += get_char_width(ord(ch))
    return res


def change_column_width(s: str, length: int) -> str:
    """
    截断字符串以适应指定宽度

    :param s: 输入字符串
    :param length: 目标宽度
    :return: 截断后的字符串
    """
    res = 0
    s_list = []
    for ch in s:
        res += get_char_width(ord(ch))
        if res <= length:
            s_list.append(ch)
    return "".join(s_list)


def dx_score(dx: int) -> int:
    """
    根据 DX 分数百分比计算星级

    :param dx: DX 分数百分比
    :return: 星级 (0-5)
    """
    if dx <= 85:
        return 0
    elif dx <= 90:
        return 1
    elif dx <= 93:
        return 2
    elif dx <= 95:
        return 3
    elif dx <= 97:
        return 4
    else:
        return 5


def image_to_bytes(img: Image.Image, format="PNG") -> bytes:
    """
    将 PIL Image 对象转换为字节流

    :param img: PIL Image 对象
    :param format: 图片格式，默认为 PNG
    :return: 图片字节流
    """
    output_buffer = BytesIO()
    img.save(output_buffer, format)
    return output_buffer.getvalue()


def rounded_corners(
    image: Image.Image, radius: int, corners: Tuple[bool, bool, bool, bool] = (False, False, False, False)
) -> Image.Image:
    """
    为图片添加圆角

    :param image: 输入图片
    :param radius: 圆角半径
    :param corners: 四个角是否应用圆角 (左上, 右上, 右下, 左下)
    :return: 处理后的图片
    """
    mask = Image.new("L", image.size, 0)
    draw = ImageDraw.Draw(mask)
    draw.rounded_rectangle((0, 0, image.size[0], image.size[1]), radius, fill=255, corners=corners)
    new_im = ImageOps.fit(image, mask.size)
    new_im.putalpha(mask)
    return new_im


def find_all_clear_rank(scores: list[PlayerMaiScore]) -> Optional[Image.Image]:
    """
    获取最小达成率并返回其对应的 rank 图
    """
    if len(scores) < 50:
        return None

    min_ach = scores[0].achievements
    for score in scores[1:]:
        if score.achievements < min_ach:
            min_ach = score.achievements

    if min_ach < 97:
        return None
    elif min_ach < 98:
        return Image.open(PIC_DIR / "UI_MSS_Allclear_Icon_S.png")
    elif min_ach < 99:
        return Image.open(PIC_DIR / "UI_MSS_Allclear_Icon_Sp.png")
    elif min_ach < 99.5:
        return Image.open(PIC_DIR / "UI_MSS_Allclear_Icon_SS.png")
    elif min_ach < 100:
        return Image.open(PIC_DIR / "UI_MSS_Allclear_Icon_SSp.png")
    elif min_ach < 100.5:
        return Image.open(PIC_DIR / "UI_MSS_Allclear_Icon_SSS.png")
    else:
        return Image.open(PIC_DIR / "UI_MSS_Allclear_Icon_SSSp.png")


class DrawText:
    """
    文本绘制辅助类
    """

    def __init__(self, image: ImageDraw.ImageDraw, font: Path) -> None:
        """
        初始化 DrawText

        :param image: ImageDraw 对象
        :param font: 字体文件路径
        """
        self._img = image
        self._font = str(font)

    def draw(
        self,
        pos_x: int,
        pos_y: int,
        size: int,
        text: Union[str, int, float],
        color: Tuple[int, int, int, int] = (255, 255, 255, 255),
        anchor: str = "lt",
        stroke_width: int = 0,
        stroke_fill: Tuple[int, int, int, int] = (0, 0, 0, 0),
        multiline: bool = False,
    ) -> None:
        """
        绘制文本

        :param pos_x: X 坐标
        :param pos_y: Y 坐标
        :param size: 字体大小
        :param text: 文本内容
        :param color: 文本颜色 (RGBA)
        :param anchor: 锚点位置
        :param stroke_width: 描边宽度
        :param stroke_fill: 描边颜色 (RGBA)
        :param multiline: 是否多行文本
        """
        font = ImageFont.truetype(self._font, size)
        if multiline:
            self._img.multiline_text(
                (pos_x, pos_y), str(text), color, font, anchor, stroke_width=stroke_width, stroke_fill=stroke_fill
            )
        else:
            self._img.text(
                (pos_x, pos_y), str(text), color, font, anchor, stroke_width=stroke_width, stroke_fill=stroke_fill
            )



"""first revision

迁移 ID: 82115b280b5c
父迁移:
创建时间: 2025-10-23 11:48:37.362789

"""

from __future__ import annotations

from collections.abc import Sequence

import sqlalchemy as sa
from alembic import op

revision: str = "82115b280b5c"
down_revision: str | Sequence[str] | None = None
branch_labels: str | Sequence[str] | None = ("nonebot_plugin_rikka",)
depends_on: str | Sequence[str] | None = None


def upgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "nonebot_plugin_rikka_userbindinfo",
        sa.Column("user_id", sa.String(), nullable=False),
        sa.Column("friend_code", sa.String(), nullable=True),
        sa.Column("lxns_api_key", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("user_id", name=op.f("pk_nonebot_plugin_rikka_userbindinfo")),
        info={"bind_key": "nonebot_plugin_rikka"},
    )
    # ### end Alembic commands ###


def downgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("nonebot_plugin_rikka_userbindinfo")
    # ### end Alembic commands ###



from __future__ import annotations

import json
from asyncio import sleep
from dataclasses import fields
from pathlib import Path

from aiohttp import ClientResponseError, ClientSession
from nonebot import logger
from nonebot_plugin_orm import async_scoped_session
from typing_extensions import TypedDict

from ..config import config
from ..constants import USER_AGENT
from ..database import MaiSongORM
from ..models.song import MaiSong, SongDifficulties

_BASE_SONG_QUERY_URL = "https://maimai.lxns.net/api/v0/maimai/song/{song_id}"
_DIVING_FISH_CHART_STATS_URL = "https://www.diving-fish.com/api/maimaidxprober/chart_stats"

_MUSIC_CHART_PATH = Path(config.static_resource_path) / "music_chart.json"

_music_chart_updated: bool = False


async def update_local_chart_file():
    """
    从水鱼更新 `music_chart.json` 文件
    """
    _MUSIC_CHART_PATH.parent.mkdir(parents=True, exist_ok=True)

    async with ClientSession() as session:
        async with session.get(_DIVING_FISH_CHART_STATS_URL, headers={"User-Agent": USER_AGENT}) as resp:
            resp.raise_for_status()
            content = await resp.json()

    _MUSIC_CHART_PATH.write_text(json.dumps(content, ensure_ascii=False, indent=4), encoding="utf-8")
    logger.info("已更新本地 music_chart.json 文件")

    global _MUSIC_CHART_DATA, _music_chart_updated
    _MUSIC_CHART_DATA = content
    _music_chart_updated = True


if not _MUSIC_CHART_PATH.exists():
    import asyncio

    asyncio.run(update_local_chart_file())

_MUSIC_CHART_DATA: "MusicChart" = json.loads(_MUSIC_CHART_PATH.read_text(encoding="utf-8"))


class MusicChartInfo(TypedDict):
    cnt: float
    diff: str
    """难度标级"""
    fit_diff: float
    """拟合定数"""
    avg: float
    """平均达成率"""
    avg_dx: float
    """平均 DX 分数"""
    std_dev: float
    dist: list[int]
    fc_dist: list[int]


class MusicChart(TypedDict):
    charts: dict[str, list[MusicChartInfo]]
    diff_data: dict[str, dict]


class MusicAliasResponseItem(TypedDict):
    song_id: int
    aliases: list[str]


class LXNSApiAliasResponse(TypedDict):
    aliases: list[MusicAliasResponseItem]


def get_song_fit_diff_from_local(song_id: int, difficulty: int) -> float:
    """
    从 `music_chart.json` 获取拟合定数

    :param song_id: 铺面ID
    :param difficulty: 难度类别
    """
    if not (chart_infos := _MUSIC_CHART_DATA["charts"].get(str(song_id))):
        raise ValueError(f"所请求的铺面 ID: {song_id} 不存在")

    elif len(chart_infos) < difficulty:
        raise ValueError(f"难度 {difficulty} 在铺面 {song_id} 中不存在")

    elif "fit_diff" not in chart_infos[difficulty]:
        raise ValueError(f"难度 {difficulty} 在铺面 {song_id} 中不存在拟合定数")

    return chart_infos[difficulty]["fit_diff"]


async def _update_song_fit_diff(difficulties: SongDifficulties, song_id: int):
    """
    更新曲目的拟合定数

    :param difficulties: 曲目的难度信息
    :param song_id: 曲目 ID
    """
    try:
        for index, difficulty in enumerate(difficulties.standard):
            difficulty.level_fit = get_song_fit_diff_from_local(song_id, index)
        for index, difficulty in enumerate(difficulties.dx):
            difficulty.level_fit = get_song_fit_diff_from_local(song_id + 10000, index)
    except ValueError:
        if _music_chart_updated:
            logger.error(f"曲目 {song_id} 的拟合定数获取失败，跳过该曲目拟合定数设置")
            return

        logger.warning(f"曲目 {song_id} 的拟合定数获取失败，尝试更新本地 chart 文件")
        await update_local_chart_file()
        await sleep(0.1)  # 避免重复请求过快
        try:
            for index, difficulty in enumerate(difficulties.standard):
                difficulty.level_fit = get_song_fit_diff_from_local(song_id, index)
            for index, difficulty in enumerate(difficulties.dx):
                difficulty.level_fit = get_song_fit_diff_from_local(song_id + 10000, index)
        except ValueError:
            logger.error(f"曲目 {song_id} 的拟合定数获取失败，跳过该曲目拟合定数设置")


async def fetch_song_info(song_id: int, interval: float = 0.3) -> MaiSong:
    """
    获取曲目信息

    :param song_id: 曲目 ID
    :param interval: 请求间隔时间

    :raise ValueError: 指定的乐曲信息不存在
    :raise ClientResponseError: 上游服务出现问题
    """
    url = _BASE_SONG_QUERY_URL.format(song_id=song_id)

    try:
        async with ClientSession() as session:
            async with session.get(url, headers={"User-Agent": USER_AGENT}) as resp:
                resp.raise_for_status()
                content = await resp.json()
                await sleep(interval)  # 避免请求过于频繁
    except ClientResponseError as exc:
        if exc.code == 404:
            raise ValueError("指定的乐曲信息不存在!")

        logger.warning(f"上游服务出现了问题无法获取曲目信息: {exc}")
        raise

    song_info_fields = {f.name for f in fields(MaiSong)}
    song_info_dict = {k: v for k, v in content.items() if k in song_info_fields}
    song_info_dict["difficulties"] = SongDifficulties.init_from_dict(content.get("difficulties", {}))

    await _update_song_fit_diff(song_info_dict["difficulties"], song_id)

    song_info = MaiSong(**song_info_dict)

    return song_info


async def update_song_alias_list(db_session: async_scoped_session):
    """
    通过落雪查分器更新别名表
    """
    _BASE_URL = "https://maimai.lxns.net/api/v0/maimai/alias/list"
    async with ClientSession() as session:
        async with session.get(_BASE_URL, headers={"User-Agent": USER_AGENT}) as resp:
            resp.raise_for_status()
            content: LXNSApiAliasResponse = await resp.json()

    from ..database import MaiSongAliasORM

    await MaiSongAliasORM.add_alias_batch(db_session, content["aliases"])


async def update_song_database(db_session: async_scoped_session) -> int:
    """
    通过落雪查分器更新曲目数据库

    :param db_session: 数据库会话对象
    :type db_session: async_scoped_session

    :return: 更新的曲目数量
    :rtype: int
    """
    _BASE_URL = "https://maimai.lxns.net/api/v0/maimai/song/list?notes=true"

    async with ClientSession() as session:
        async with session.get(_BASE_URL, headers={"User-Agent": USER_AGENT}) as resp:
            resp.raise_for_status()
            content = await resp.json()

    songs = content["songs"]
    songs_obj = []
    song_info_fields = {f.name for f in fields(MaiSong)}

    for song in songs:
        song_info_dict = {k: v for k, v in song.items() if k in song_info_fields}
        song_info_dict["difficulties"] = SongDifficulties.init_from_dict(song.get("difficulties", {}))

        # 获取拟合定数
        await _update_song_fit_diff(song_info_dict["difficulties"], song_info_dict["id"])

        song_info = MaiSong(**song_info_dict)
        songs_obj.append(song_info)

    await MaiSongORM.save_song_info_batch(db_session, songs_obj)

    return len(songs_obj)


async def get_plate_data() -> dict:
    """
    获取牌子数据
    """
    _BASE_URL = "https://www.yuzuchan.moe/api/maimaidx/maimaidxplate"

    async with ClientSession() as session:
        async with session.get(_BASE_URL, headers={"User-Agent": USER_AGENT}) as resp:
            resp.raise_for_status()
            content = await resp.json()

    data = content["content"]

    return data



from __future__ import annotations

import importlib
import inspect
import re
import sys
from dataclasses import asdict, is_dataclass
from pathlib import Path
from typing import Any, Mapping, Optional

DEV_MODULE_PATH = Path("./nonebot-plugin-rikka-extra")
if DEV_MODULE_PATH.exists():
    sys.path.insert(0, str(DEV_MODULE_PATH))


class ExtraNotInstalledError(RuntimeError):
    """Raised when nonebot-plugin-rikka-extra is not installed."""


def _load_extra_attr(module_name: str, attr: str) -> Any:
    try:
        mod = importlib.import_module(f"nonebot_plugin_rikka_extra.{module_name}")
    except ModuleNotFoundError as e:
        raise ExtraNotInstalledError("未安装 nonebot-plugin-rikka-extra") from e

    try:
        return getattr(mod, attr)
    except AttributeError as e:
        raise RuntimeError(f"nonebot-plugin-rikka-extra 缺少接口: {attr}") from e


async def run_extend_score_workflow(qr_code: str) -> list[dict[str, Any]]:
    """Proxy to nonebot-plugin-rikka-extra.score.run_workflow"""

    workflow = _load_extra_attr("score", "run_workflow")
    result = workflow(qr_code)
    if inspect.isawaitable(result):
        result = await result
    if not isinstance(result, list):
        raise RuntimeError("run_workflow 返回结果格式异常")
    return result


async def run_extend_ticket_workflow(qr_code: str) -> None:
    """Proxy to nonebot-plugin-rikka-extra.ticket.run_workflow"""

    workflow = _load_extra_attr("ticket", "run_workflow")
    result = workflow(qr_code)
    if inspect.isawaitable(result):
        await result


async def run_unlock_workflow(qr_code: str) -> None:
    """Proxy to nonebot-plugin-rikka-extra.unlock_all.run_workflow"""

    workflow = _load_extra_attr("unlock_all", "run_workflow")
    result = workflow(qr_code)
    if inspect.isawaitable(result):
        await result


async def run_divingfish_import_workflow(qr_code: str, import_token: str) -> None:
    """Proxy to nonebot-plugin-rikka-extra.Best50_To_Diving_Fish.run_workflow"""

    workflow = _load_extra_attr("convert", "run_workflow")
    result = workflow(qr_code, import_token)
    if inspect.isawaitable(result):
        await result


async def run_extent_force_logout(qr_code: str):
    func = _load_extra_attr("logout", "force_logout")
    result = func(qr_code)
    if inspect.isawaitable(result):
        await result


async def get_maistatus(
    *,
    title_ver: str = "1.51",
    initialize_payload: Optional[str] = None,
    timeout_seconds: float = 20.0,
) -> str:
    """Proxy to nonebot_plugin_rikka_extra.server_status.check_allnet_server_status.

    Returns a human-readable text summary, so callers don't need to care about
    extra package's dataclass shapes.
    """

    status = await _check_allnet_server_status_obj(
        title_ver=title_ver,
        initialize_payload=initialize_payload,
        timeout_seconds=timeout_seconds,
    )
    data = _status_obj_to_dict(status)
    return format_allnet_server_status(data)


async def _check_allnet_server_status_obj(
    *,
    title_ver: str,
    initialize_payload: Optional[str],
    timeout_seconds: float,
) -> Any:
    check_func = _load_extra_attr("server_status", "check_allnet_server_status")
    return await check_func(
        title_ver=title_ver,
        initialize_payload=initialize_payload,
        timeout_seconds=timeout_seconds,
    )


def _status_obj_to_dict(status: Any) -> dict[str, Any]:
    if hasattr(status, "to_dict"):
        data = status.to_dict()
        if isinstance(data, dict):
            return data

    if is_dataclass(status) and not isinstance(status, type):
        return asdict(status)

    if isinstance(status, dict):
        return dict(status)

    return {"value": status}


def _get_result_code(text: Optional[str]) -> Optional[str]:
    if not text:
        return None
    m = re.search(r"(?:^|&)result=([^&\s]+)", text)
    return m.group(1) if m else None


def _fmt_elapsed_ms(value: Any) -> str:
    try:
        if value is None:
            return "-"
        return f"{float(value):.0f}ms"
    except Exception:
        return "-"


def format_allnet_server_status(status: Mapping[str, Any]) -> str:
    """Format server status dict into a human-readable summary."""

    overall_ok = bool(status.get("ok"))
    header = f"所有服务器状态: {'正常' if overall_ok else '异常'}"

    delivery = status.get("delivery") or {}
    init = status.get("initialize") or {}
    aime = status.get("aime") or {}
    title = status.get("title") or {}

    delivery_ok = bool(delivery.get("ok"))
    delivery_elapsed = _fmt_elapsed_ms(delivery.get("elapsed_ms"))
    # Hide low-level details like result codes by default.

    init_ok = bool(init.get("ok"))
    init_elapsed = _fmt_elapsed_ms(init.get("elapsed_ms"))
    # Hide low-level details like status_code/result by default.

    aime_ok = bool(aime.get("ok"))
    aime_elapsed = _fmt_elapsed_ms(aime.get("elapsed_ms"))

    title_ok = bool(title.get("ok"))
    title_elapsed = _fmt_elapsed_ms(title.get("elapsed_ms"))
    title_empty = bool(title.get("empty_response"))

    lines: list[str] = [header]

    delivery_line = f"- 舞萌更新服务(Net delivery): {'正常' if delivery_ok else '异常'} ({delivery_elapsed})"
    lines.append(delivery_line)

    init_line = f"- 机台启动服务(NET Initialize): {'正常' if init_ok else '异常'} ({init_elapsed})"
    lines.append(init_line)

    aime_line = f"- 舞萌登录服务(Chime): {'正常' if aime_ok else '异常'} ({aime_elapsed})"
    lines.append(aime_line)

    title_line = f"- 舞萌游戏服务(Title): {'正常' if title_ok else '异常'} ({title_elapsed})"
    if title_ok and title_empty:
        title_line += " | 空响应（可能是 Bot 所属服务器 IP 被限制请求）"
    elif not title_ok and title_elapsed > "5000ms":
        title_line += " | 响应超时"
    lines.append(title_line)

    return "\n".join(lines)



from typing import Optional

from nonebot_plugin_orm import Model
from sqlalchemy import ForeignKey, Integer, String
from sqlalchemy.orm import Mapped, mapped_column, relationship
from typing_extensions import Literal, TypedDict


class SongDifficulties(TypedDict):
    standard: list
    """曲目标准谱面难度列表"""
    dx: list
    """曲目 DX 谱面难度列表"""
    utage: list
    """可选，宴会场曲目谱面难度列表"""


class UserBindInfo(Model):
    user_id: Mapped[str] = mapped_column(primary_key=True)
    default_provider: Mapped[Literal["lxns", "divingfish"]] = mapped_column(String, nullable=True, default="lxns")
    friend_code: Mapped[str] = mapped_column(String, nullable=True, default="")
    lxns_api_key: Mapped[Optional[str]] = mapped_column(String, nullable=True, default="")
    diving_fish_import_token: Mapped[Optional[str]] = mapped_column(String, nullable=True, default="")
    diving_fish_username: Mapped[Optional[str]] = mapped_column(String, nullable=True, default="")
    maimaipy_identifier: Mapped[Optional[str]] = mapped_column(String, nullable=True, default="")


class MaiSong(Model):
    __tablename__ = "nonebot_plugin_rikka_maisong"  # nonebot_plugin_orm 的默认命名

    id: Mapped[int] = mapped_column(primary_key=True)
    title: Mapped[str] = mapped_column(String, nullable=False)
    artist: Mapped[str] = mapped_column(String, nullable=False)
    genre: Mapped[str] = mapped_column(String, nullable=False)
    bpm: Mapped[int] = mapped_column(Integer, nullable=False)
    map: Mapped[Optional[str]] = mapped_column(String, nullable=True, default="")
    version: Mapped[int] = mapped_column(Integer, nullable=False)
    locked: Mapped[bool] = mapped_column(nullable=True, default=False)
    disabled: Mapped[bool] = mapped_column(nullable=True, default=False)
    difficulties: Mapped[SongDifficulties] = mapped_column(String, nullable=False)

    alias_entry: Mapped["MaiSongAlias"] = relationship("MaiSongAlias", back_populates="song", uselist=False)


class MaiSongAlias(Model):
    song_id: Mapped[int] = mapped_column(ForeignKey("nonebot_plugin_rikka_maisong.id"), primary_key=True)
    alias: Mapped[str] = mapped_column(String, nullable=True, default="[]")
    custom_alias: Mapped[str] = mapped_column(String, nullable=True, default="[]")

    song: Mapped["MaiSong"] = relationship("MaiSong", back_populates="alias_entry")


class MaiPlayCount(Model):
    """玩家铺面游玩次数"""

    __tablename__ = "nonebot_plugin_rikka_playcount"

    user_id: Mapped[str] = mapped_column(String, primary_key=True)
    song_id: Mapped[int] = mapped_column(Integer, primary_key=True)
    difficulty: Mapped[int] = mapped_column(Integer, primary_key=True)
    play_count: Mapped[int] = mapped_column(Integer, nullable=False, default=0)



from dataclasses import dataclass, field
from enum import Enum
from typing import Optional

from maimai_py import PlayerFrame, PlayerIcon, PlayerNamePlate, PlayerTrophy


class SongDifficulty(int, Enum):
    """铺面难度"""

    BASIC = 0
    """宴会场铺面也为0"""
    ADVANCED = 1
    EXPERT = 2
    MASTER = 3
    ReMASTER = 4


class ScoreFCType(str, Enum):
    """FULL COMBO 类型"""

    APP = "app"
    """ALL PERFECT PLUS"""
    AP = "ap"
    """ALL PERFECT"""
    FCP = "fcp"
    """FULL COMBO PLUS"""
    FC = "fc"
    """FULL COMBO"""


class ScoreFSType(str, Enum):
    """FULL SYNC 类型"""

    FSDP = "fsdp"
    """FDX+"""
    FSD = "fsd"
    """FDX"""
    FSP = "fsp"
    """FS+"""
    FS = "fs"
    """FS"""
    SYNC = "sync"
    """SYNC PLAY"""


class ScoreRateType(str, Enum):
    """评级类型"""

    SSSP = "sssp"
    """SSS+"""
    SSS = "sss"
    """SSS"""
    SSP = "ssp"
    """SS+"""
    SS = "ss"
    """SS"""
    SP = "sp"
    """S+"""
    S = "s"
    """S"""
    AAA = "aaa"
    """AAA"""
    AA = "aa"
    """AA"""
    A = "a"
    """A"""
    BBB = "bbb"
    """BBB"""
    BB = "bb"
    """BB"""
    B = "b"
    """B"""
    C = "c"
    """C"""
    D = "d"
    """D"""


class TrophyColor(str, Enum):
    """称号颜色"""

    NORMAL = "Normal"
    """普通"""
    BRONZE = "Bronze"
    """铜"""
    SILVER = "Silver"
    """银"""
    GOLD = "Gold"
    """金"""
    RAINBOW = "Rainbow"
    """虹"""


class SongType(str, Enum):
    """谱面类型"""

    STANDARD = "standard"
    """标准谱面"""
    DX = "dx"
    """DX 谱面"""
    UTAGE = "utage"
    """宴会场谱面"""


@dataclass
class PlayerMaiScore:
    song_id: int
    """乐曲 ID"""
    song_name: str
    """乐曲名"""
    song_type: SongType
    """铺面类型"""
    song_level: str
    """铺面等级，如 `11+`"""
    song_difficulty: SongDifficulty
    """铺面难度分类"""
    achievements: float
    """达成率"""
    dx_score: int
    """DX 分数"""
    dx_star: int
    """DX 星级"""
    dx_rating: float
    """DX Rating"""
    rate: ScoreRateType
    """评级"""
    fc: Optional[ScoreFCType] = None
    """FC 情况"""
    fs: Optional[ScoreFSType] = None
    """同步游玩情况"""
    song_level_value: Optional[float] = None
    """乐曲定数"""
    play_count: Optional[int] = None
    """游玩次数"""


@dataclass
class PlayerMaiB50:
    """玩家B50信息，也可用于AP50等类型"""

    standard: list[PlayerMaiScore] = field(default_factory=list)
    """旧版本谱面 Best 35 列表"""
    dx: list[PlayerMaiScore] = field(default_factory=list)
    """现版本谱面 Best 15 列表"""

    @property
    def rating(self) -> int:
        rating = 0.0

        for score in self.standard[:35] + self.dx[:15]:
            rating += score.dx_rating

        return int(rating)


@dataclass
class PlayerMaiCollection:
    """玩家舞萌收藏品"""

    id: int
    """收藏品ID"""
    name: str
    """收藏品名称"""
    description: Optional[str] = None
    """收藏品说明"""
    genre: Optional[str] = None
    """收藏品分类"""


@dataclass
class PlayerMaiTrophy:
    """玩家舞萌称号"""

    id: int
    """收藏品ID"""
    name: str
    """收藏品名称"""
    color: TrophyColor
    """称号颜色"""
    description: Optional[str] = None
    """收藏品说明"""
    genre: Optional[str] = None
    """收藏品分类"""


@dataclass
class PlayerMaiInfo:
    """玩家舞萌DX信息"""

    name: str
    """游戏内名称"""
    rating: int
    """玩家 DX Rating"""
    course_rank: int = 0
    """段位 ID"""
    class_rank: int = 0
    """阶级 ID"""
    friend_code: Optional[str] = None
    """好友码"""
    trophy: Optional[PlayerTrophy] = None
    """称号"""
    icon: Optional[PlayerIcon] = None
    """头像"""
    name_plate: Optional[PlayerNamePlate] = None
    """姓名框"""
    frame: Optional[PlayerFrame] = None
    """背景"""
    upload_time: Optional[str] = None
    """成绩上传时间"""



from dataclasses import dataclass
from decimal import ROUND_DOWN, Decimal
from typing import Literal, Optional, TypeAlias, TypeVar, cast

from async_lru import alru_cache
from httpcore import NetworkError
from httpx import HTTPStatusError
from maimai_py import (
    ArcadeProvider,
    DivingFishPlayer,
    DivingFishProvider,
    FCType,
    InvalidPlayerIdentifierError,
    LXNSPlayer,
    LXNSProvider,
    MaimaiClient,
    MaimaiScores,
    Player,
    PlayerIdentifier,
    ScoreExtend,
)
from maimai_py import SongType as MaimaiPySongType
from maimai_py import (
    current_version,
)
from nonebot import logger
from nonebot.internal.matcher import current_event
from nonebot_plugin_alconna import At, UniMessage
from nonebot_plugin_orm import async_scoped_session, get_scoped_session

from ...config import config
from ...database import MaiPlayCountORM, UserBindInfo, UserBindInfoORM
from .._base import BaseScoreProvider
from .._schema import (
    PlayerMaiB50,
    PlayerMaiInfo,
    PlayerMaiScore,
    ScoreFCType,
    ScoreFSType,
    ScoreRateType,
    SongDifficulty,
    SongType,
)

_SUPPORT_PROVIDER: TypeAlias = DivingFishProvider | LXNSProvider | ArcadeProvider
S = TypeVar("S", bound=list[PlayerMaiScore] | PlayerMaiB50)

maimai_client = MaimaiClient()
_divingfish_provider = DivingFishProvider(developer_token=config.divingfish_developer_api_key)
_lxns_provider = LXNSProvider(developer_token=config.lxns_developer_api_key)
_arcade_provider = ArcadeProvider(http_proxy=config.arcade_provider_http_proxy)


@dataclass
class MaimaiPyParams:
    score_provider: _SUPPORT_PROVIDER
    identifier: PlayerIdentifier


class MaimaiPyScoreProvider(BaseScoreProvider[MaimaiPyParams]):
    provider = "maimai_py"
    ParamsType = MaimaiPyParams

    @staticmethod
    def _score_unpack(score: ScoreExtend) -> PlayerMaiScore:
        return PlayerMaiScore(
            song_id=score.id,
            song_name=score.title,
            song_type=SongType(score.type.value),
            song_level=score.level,
            song_difficulty=SongDifficulty(score.level_index.value),
            achievements=score.achievements,  # type: ignore
            dx_score=score.dx_score or 0,
            dx_star=0,  # unsupported.
            dx_rating=score.dx_rating or 0,
            rate=ScoreRateType(score.rate.name.lower()),
            fc=ScoreFCType(score.fc.name.lower()) if score.fc else None,
            fs=ScoreFSType(score.fs.name.lower()) if score.fs else None,
            play_count=score.play_count,
        )

    @staticmethod
    def _unpack_player_mai_info(player: Player | LXNSPlayer | DivingFishPlayer) -> "PlayerMaiInfo":
        if isinstance(player, LXNSPlayer):
            player_info = PlayerMaiInfo(
                name=player.name,
                rating=player.rating,
                course_rank=player.course_rank,
                class_rank=player.class_rank,
                friend_code=str(player.friend_code),
                trophy=player.trophy,
                icon=player.icon,
                name_plate=player.name_plate,
                frame=player.frame,
                upload_time=player.upload_time,
            )
        else:
            player_info = PlayerMaiInfo(
                name=player.name,
                rating=player.rating,
            )

        return player_info

    @staticmethod
    async def auto_get_score_provider(session: async_scoped_session, user_id: str) -> _SUPPORT_PROVIDER:
        """
        根据用户绑定情况自动获取合适的查分器

        选择逻辑如下：
        1. 如果仅配置了一个查分器的开发者密钥，则使用该查分器
        2. 如果用户未绑定任何查分器，默认使用 LXNS 查分器
        3. 如果用户绑定了查分器但未设置默认查分器，优先使用水鱼查分器（如果有绑定水鱼查分器），否则使用 LXNS 查分器
        """
        bind_info = await UserBindInfoORM.get_user_bind_info(session, user_id)

        if config.lxns_developer_api_key and not config.divingfish_developer_api_key:
            return _lxns_provider
        if config.divingfish_developer_api_key and not config.lxns_developer_api_key:
            return _divingfish_provider

        if not bind_info:
            return _lxns_provider

        if bind_info.default_provider is None:
            if bind_info.diving_fish_import_token:
                return _divingfish_provider
            else:
                return _lxns_provider

        if bind_info.default_provider == "lxns":
            return _lxns_provider

        elif bind_info.default_provider == "divingfish":
            return _divingfish_provider

        return _lxns_provider

    @staticmethod
    async def auto_get_player_identifier(
        session: async_scoped_session,
        user_id: str,
        score_provider: _SUPPORT_PROVIDER,
        *,
        use_personal_api: bool = False,
    ) -> PlayerIdentifier:
        """
        根据用户绑定情况自动选择鉴权方式
        """
        logger.debug(f"[{user_id}] 尝试从数据库中获取玩家绑定信息...")
        user_bind_info = await UserBindInfoORM.get_user_bind_info(session, user_id)

        # 落雪查分器
        # When user_bind_info is None, score_provider is LXNSScoreProvider.
        if isinstance(score_provider, LXNSProvider):
            new_player_friend_code = None
            if use_personal_api:
                if user_bind_info:
                    return PlayerIdentifier(credentials=user_bind_info.lxns_api_key)
                else:
                    raise ValueError("无法通过个人 API 获得用户鉴权凭证：用户未绑定落雪查分器")

            if user_bind_info is None:
                logger.warning(f"[{user_id}] 未能获取玩家码，数据库中不存在绑定的玩家数据")
                logger.debug(f"[{user_id}] 尝试通过 QQ 请求玩家数据")
                try:
                    identifier = PlayerIdentifier(qq=int(user_id))
                    player_obj = await score_provider.get_player(identifier, client=maimai_client)
                    new_player_friend_code = player_obj.friend_code
                except (HTTPStatusError, InvalidPlayerIdentifierError) as e:
                    if isinstance(e, HTTPStatusError) and e.response.status_code == 404:
                        logger.warning(f"[{user_id}] 无法通过 QQ 号请求玩家数据: 玩家不存在")
                    else:
                        logger.warning(f"[{user_id}] 无法通过 QQ 号请求玩家数据: {e}")

                    await UniMessage(
                        [
                            At(flag="user", target=user_id),
                            "你还未绑定查分器，请使用 /bind 指令进行绑定！",
                        ]
                    ).finish()

                    return
                except ValueError:
                    logger.warning(f"[{user_id}] 无法通过 QQ 号请求玩家数据: QQ 号无效 & 非 QQ 平台聊天")
                    await UniMessage(
                        [
                            At(flag="user", target=user_id),
                            "你还未绑定查分器，请使用 /bind 指令进行绑定！",
                        ]
                    ).finish()

                    return

            friend_code = new_player_friend_code or user_bind_info.friend_code  # type: ignore
            if not friend_code:
                logger.warning(f"[{user_id}] 无法获取好友码，无法继续查询。")
                await UniMessage(
                    [
                        At(flag="user", target=user_id),
                        "无法获取好友码，请确认已绑定或查分器可用。",
                    ]
                ).finish()
                return

            identifier = PlayerIdentifier(friend_code=int(friend_code))

        # 水鱼查分器
        # elif isinstance(score_provider, DivingFishScoreProvider):
        else:
            # When score_provider is DivingFish, user_bind_info and diving_fish_username is not None.
            user_bind_info = cast(UserBindInfo, user_bind_info)
            diving_fish_username = user_bind_info.diving_fish_username
            assert diving_fish_username
            identifier = PlayerIdentifier(
                username=diving_fish_username, credentials=user_bind_info.diving_fish_import_token
            )

        return identifier

    async def fetch_player_info(self, params: MaimaiPyParams) -> PlayerMaiInfo:
        player_info = await maimai_client.players(params.identifier, params.score_provider)

        return self._unpack_player_mai_info(player_info)

    async def fetch_player_b50(self, params: MaimaiPyParams) -> PlayerMaiB50:
        player_b50 = await maimai_client.bests(params.identifier, params.score_provider)

        best35 = [self._score_unpack(score) for score in player_b50.scores_b35]
        best15 = [self._score_unpack(score) for score in player_b50.scores_b15]

        return await self.fetch_player_play_counts(PlayerMaiB50(best35, best15))

    async def fetch_player_ap50(self, params: MaimaiPyParams) -> PlayerMaiB50:
        """
        获得玩家 AP 50

        注: 落雪查分器需要使用个人 token 进行鉴权
        """
        logger.debug("1/2 获取完整游玩记录")

        scores = await maimai_client.scores(params.identifier, params.score_provider)

        ap_records = scores.filter(fc=FCType.AP) + scores.filter(fc=FCType.APP)

        logger.debug("2/2 划分版本信息")

        ap50 = await MaimaiScores(maimai_client).configure(ap_records, b50_only=True)

        best35 = [self._score_unpack(score) for score in ap50.scores_b35]
        best15 = [self._score_unpack(score) for score in ap50.scores_b15]

        return await self.fetch_player_play_counts(PlayerMaiB50(best35, best15))

    async def fetch_player_pc50(self, params: MaimaiPyParams) -> PlayerMaiB50:
        """
        获得玩家 PC 50

        注：需要用户使用 `.import` 指令导入游玩记录后才能获取
        """
        logger.debug("1/3 获取完整游玩记录")
        scores = await maimai_client.scores(params.identifier, params.score_provider)

        scores_list = scores.scores.copy()
        # scores_list.sort(key=lambda x: x.play_count or 0, reverse=True)

        logger.debug("2/3 划分版本信息")
        old_version_scores: list[ScoreExtend] = []
        new_version_scores: list[ScoreExtend] = []
        song_diff_versions: dict[str, int] = await maimai_client._cache.get("versions", namespace="songs") or {}
        for score in scores_list:
            if score.type == MaimaiPySongType.UTAGE:
                continue

            # Find the version of the song, and decide whether it is b35 or b15.
            diff_key = f"{score.id} {score.type} {score.level_index}"
            if score_version := song_diff_versions.get(diff_key, None):
                (new_version_scores if score_version >= current_version.value else old_version_scores).append(score)

        logger.debug("3/3 获取游玩次数并返回结果")
        old_best = await self.fetch_player_play_counts([self._score_unpack(score) for score in old_version_scores])
        new_best = await self.fetch_player_play_counts([self._score_unpack(score) for score in new_version_scores])

        best35 = sorted(old_best, key=lambda x: x.play_count or 0, reverse=True)[:35]
        best15 = sorted(new_best, key=lambda x: x.play_count or 0, reverse=True)[:15]

        return PlayerMaiB50(standard=best35, dx=best15)

    async def fetch_player_minfo(
        self, params: MaimaiPyParams, song_id: int, song_type: Literal["standard", "dx"]
    ) -> list[PlayerMaiScore]:
        """
        获取玩家单曲游玩情况

        :param params: 鉴权参数对象
        :type params: MaimaiPyParams
        :param name: 曲目的 ID（标准曲目ID）
        :type name: str
        :param song_type: 铺面类型
        :type song_type: Literal["standard", "dx"]
        :return: 成绩列表
        :rtype: list[PlayerMaiScore]
        """

        player_song = await maimai_client.minfo(song_id, params.identifier, params.score_provider)

        if not player_song:
            return []

        raw_scores = player_song.scores
        scores = [self._score_unpack(score) for score in raw_scores if score.type.value == song_type]

        return await self.fetch_player_play_counts(scores)

    async def fetch_player_scoreslist(
        self,
        params: MaimaiPyParams,
        level: Optional[str] = None,
        ach: Optional[float] = None,
        diff: Optional[Literal["BASIC", "ADVANCED", "EXPERT", "MASTER", "REMASTER"]] = None,
    ) -> list[PlayerMaiScore]:
        """
        获得玩家指定条件的成绩列表

        :param params: 鉴权参数对象
        :type params: MaimaiPyParams
        :param level: 曲目等级，如 `11`, `12+`
        :type level: str
        :param ach: 达成率，精确一位小数
        :type ach: float
        :param diff: 铺面难度分类
        :type diff: Literal["BASIC", "ADVANCED", "EXPERT", "MASTER", "REMASTER"]

        :return: 成绩列表
        :rtype: list[PlayerMaiScore]
        """

        def trunc_1(x):
            return Decimal(str(x)).quantize(Decimal("0.0"), rounding=ROUND_DOWN)

        scores = await maimai_client.scores(params.identifier, params.score_provider)
        matched_scores: list[ScoreExtend] = []

        if level:
            matched_scores = cast(list[ScoreExtend], scores.filter(level=level))
        elif ach:
            for score in scores.scores:
                if score.achievements and trunc_1(score.achievements) == trunc_1(ach):
                    matched_scores.append(score)
        elif diff:
            for score in scores.scores:
                if score.level_index.name == diff:
                    matched_scores.append(score)
        else:
            matched_scores = scores.scores

        matched_scores.sort(key=lambda x: x.achievements or 0.0, reverse=True)

        return await self.fetch_player_play_counts([self._score_unpack(score) for score in matched_scores])

    async def get_player_identifier(self, qr_code_data: str) -> str:
        """
        获取玩家 Arcade 鉴权凭证（加密）

        :param qr_code_data: 仍在有效期内的二维码数据
        :type qr_code_data: str
        :return: Arcade 加密鉴权凭证
        :rtype: str
        """
        try:
            player = await maimai_client.identifiers(qr_code_data, provider=_arcade_provider)
        except NetworkError as exc:
            logger.warning(f"尝试通过玩家二维码获取玩家数据时发生错误: {exc}")
            raise

        return player.credentials  # type: ignore

    @alru_cache(maxsize=128, ttl=60 * 60)
    async def _fetch_player_play_counts(self, identifier: str) -> list[PlayerMaiScore]:
        """
        获取用户各铺面游玩次数
        """
        maimai_scores = await maimai_client.scores(PlayerIdentifier(credentials=identifier), _arcade_provider)

        return [self._score_unpack(score) for score in maimai_scores.scores]

    async def fetch_player_play_counts(self, scores: S) -> S:
        """
        根据玩家成绩补充游玩次数
        """
        session = get_scoped_session()
        try:
            user_id = current_event.get().get_user_id()
        except LookupError:
            # 当事件上下文不存在时，无法获取用户 ID，直接返回原始成绩列表
            return scores

        score_list = scores if isinstance(scores, list) else scores.dx + scores.standard
        if not score_list:
            return scores

        song_ids = [
            (score.song_id + 10000 if score.song_type == SongType.DX else score.song_id) for score in score_list
        ]
        play_map = await MaiPlayCountORM.get_user_play_count_map(session, user_id, song_ids)

        for score in score_list:
            song_key = score.song_id + 10000 if score.song_type == SongType.DX else score.song_id
            play_count = play_map.get((song_key, score.song_difficulty.value))
            if play_count is not None:
                score.play_count = play_count

        return scores



from __future__ import annotations

from typing import TypedDict

from ..browser import get_browser, get_page_semaphore


class Viewport(TypedDict):
    width: int
    height: int


async def capture_webpage_png(
    url: str,
    *,
    viewport: Viewport = Viewport(width=1400, height=900),
    full_page: bool = True,
    timeout_ms: int = 30_000,
    wait_ms: int = 500,
) -> bytes:
    """Capture a webpage screenshot as PNG bytes.

    Notes:
        - This relies on Playwright + Chromium.
        - If you see errors about missing browser executables, run:
          `python -m playwright install chromium`
    """
    if not url.startswith("http://") and not url.startswith("https://"):
        raise ValueError("url 必须以 http:// 或 https:// 开头")

    page_semaphore = await get_page_semaphore()
    async with page_semaphore:
        browser = await get_browser()
        context = await browser.new_context(viewport=viewport)
        try:
            page = await context.new_page()
            await page.goto(url, wait_until="networkidle", timeout=timeout_ms)
            if wait_ms > 0:
                await page.wait_for_timeout(wait_ms)
            return await page.screenshot(
                type="png",
                full_page=full_page,
                clip=(
                    None
                    if full_page
                    else {
                        "x": 0,
                        "y": 0,
                        "width": viewport["width"],
                        "height": viewport["height"],
                    }
                ),
            )
        finally:
            await context.close()


async def capture_maimai_status_png(maimai_status_url: str) -> bytes:
    """Capture maimai status page screenshot."""

    return await capture_webpage_png(
        maimai_status_url,
        viewport={"width": 1200, "height": 850},
        full_page=False,
        timeout_ms=20_000,
        wait_ms=800,
    )



"""add play count

迁移 ID: dc59349a578a
父迁移: 5c791c303fe1
创建时间: 2026-01-24 09:19:05.905902

"""

from __future__ import annotations

from collections.abc import Sequence

import sqlalchemy as sa
from alembic import op

revision: str = "dc59349a578a"
down_revision: str | Sequence[str] | None = "5c791c303fe1"
branch_labels: str | Sequence[str] | None = None
depends_on: str | Sequence[str] | None = None


def upgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "nonebot_plugin_rikka_playcount",
        sa.Column("user_id", sa.String(), nullable=False),
        sa.Column("song_id", sa.Integer(), nullable=False),
        sa.Column("difficulty", sa.Integer(), nullable=False),
        sa.Column("play_count", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint(
            "user_id",
            "song_id",
            "difficulty",
            name=op.f("pk_nonebot_plugin_rikka_playcount"),
        ),
        info={"bind_key": "nonebot_plugin_rikka"},
    )
    # ### end Alembic commands ###


def downgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("nonebot_plugin_rikka_playcount")
    # ### end Alembic commands ###



import os
from datetime import datetime
from tempfile import TemporaryFile
from typing import List, Literal, Optional, Sequence, Tuple

import matplotlib.dates as mdates
import matplotlib.pyplot as plt
import numpy as np
from PIL import Image

from ..config import config
from ..score.providers.lxns import LXNSRatingTrend

_DEFAULT_FONT = "SimHei"
plt.rcParams["font.sans-serif"] = [_DEFAULT_FONT]
plt.rcParams["axes.unicode_minus"] = False


def _format_trend_date_labels(dates: Sequence[str]) -> Tuple[List[str], bool]:
    """将日期字符串格式化为更短的刻度标签（优先 %m-%d）。

    返回 (labels, all_parsed_ok)。
    """

    labels: List[str] = []
    ok = True
    for d in dates:
        try:
            dt = datetime.fromisoformat(d.replace("Z", "+00:00"))
            labels.append(dt.strftime("%m-%d"))
        except ValueError:
            ok = False
            labels.append(d)
    return labels, ok


def _parse_trend_dates(dates: Sequence[str]) -> Tuple[List[datetime], bool]:
    """将日期字符串解析为 datetime 列表，用于真实时间轴。"""

    parsed: List[datetime] = []
    ok = True
    for d in dates:
        try:
            parsed.append(datetime.fromisoformat(d.replace("Z", "+00:00")))
        except ValueError:
            ok = False
            # 占位，最终会 fallback 到 index 模式
            parsed.append(datetime.fromtimestamp(0))
    return parsed, ok


def draw_player_rating_trend(
    trends: Sequence[LXNSRatingTrend],
    *,
    title: Optional[str] = None,
    show_standard_dx: bool = True,
    x_axis: Literal["index", "time"] = "time",
) -> Image.Image:
    """绘制玩家 RatingTrend 折线图。

    :param trends: 趋势数据列表（包含 total/standard/dx/date）
    :param title: 图表标题
    :param show_standard_dx: 是否同时绘制 standard/dx 两条分量线
    """

    # 按 date 排序
    sorted_trends = sorted(trends, key=lambda t: str(t.get("date", "")))
    dates = [t["date"] for t in sorted_trends]
    totals = [int(t["total"]) for t in sorted_trends]
    standards = [int(t["standard_total"]) for t in sorted_trends]
    dxs = [int(t["dx_total"]) for t in sorted_trends]

    # x 轴：
    # - index：每条数据等间隔（适合“数据点很多，只看走势”，不会被日期空档拉开）
    # - time：按真实日期间隔（适合“希望空档可见”，但视觉上会出现间距不均）
    x_values: np.ndarray
    labels, _ = _format_trend_date_labels(dates)
    if x_axis == "time":
        parsed_dates, ok = _parse_trend_dates(dates)
        if ok:
            # mdates.date2num -> ndarray[float]，满足类型检查也可用于 fill_between
            x_values = np.asarray(mdates.date2num(parsed_dates), dtype=float)
        else:
            x_values = np.arange(len(sorted_trends), dtype=float)
            x_axis = "index"
    else:
        x_values = np.arange(len(sorted_trends), dtype=float)

    bg_path = config.static_resource_path + "/mai/pic/UI_TTR_BG_Base.png"
    dpi = 150
    bg_size: tuple[int, int] | None = None
    if os.path.exists(bg_path):
        try:
            with Image.open(bg_path) as _bg_probe:
                bg_size = _bg_probe.size
        except Exception:
            bg_size = None

    # 如果有背景图，则让画布尺寸与背景像素尺寸一致，这样不会拉伸背景。
    if bg_size:
        fig = plt.figure(figsize=(bg_size[0] / dpi, bg_size[1] / dpi), dpi=dpi)
    else:
        fig = plt.figure(figsize=(18, 6), dpi=dpi)
    ax = fig.add_subplot(111)

    # 先给一个基础边距；后面会根据 y 轴数值位数自适应加大 left，避免刻度数字被裁剪。
    fig.subplots_adjust(left=0.06, right=0.98, top=0.92, bottom=0.18)

    if title:
        ax.set_title(title, fontsize=18, fontweight="bold", color="#000000", family=_DEFAULT_FONT, pad=14)

    # 颜色与现有 analysis 保持同一套主色
    c_total = "#4ECDC4"
    c_std = "#5B8FF9"
    c_dx = "#F6BD16"

    # y 轴范围：上下留白，避免贴边
    y_min = min(totals + (standards + dxs if show_standard_dx else []))
    y_max = max(totals + (standards + dxs if show_standard_dx else []))
    pad = max(10, int((y_max - y_min) * 0.08))
    y_base = y_min - pad
    ax.set_ylim(y_base, y_max + pad)

    # 不使用 bbox_inches='tight' 保存时，超出画布的 ticklabel 会被裁剪。
    # 这里根据 y 轴数值位数动态加大 left 边距（常见 5 位数时会被截断最后一位）。
    y_digits = max(len(str(abs(int(y_base)))), len(str(abs(int(y_max + pad)))))
    extra_left = max(0.0, (y_digits - 4) * 0.02)
    fig.subplots_adjust(left=min(0.18, 0.06 + extra_left), right=0.98, top=0.92, bottom=0.18)

    # 带填充的折线图：数据多时不绘制端点（无 marker）
    ax.fill_between(x_values, totals, y_base, color=c_total, alpha=0.18, linewidth=0)
    ax.plot(x_values, totals, linewidth=2.2, color=c_total, label="Rating")
    if show_standard_dx:
        ax.fill_between(x_values, standards, y_base, color=c_std, alpha=0.10, linewidth=0)
        ax.plot(x_values, standards, linewidth=1.6, color=c_std, alpha=0.95, label="旧版本铺面")
        ax.fill_between(x_values, dxs, y_base, color=c_dx, alpha=0.10, linewidth=0)
        ax.plot(x_values, dxs, linewidth=1.6, color=c_dx, alpha=0.95, label="新版本铺面")

    ax.grid(True, linestyle="--", alpha=0.28, color="#CCCCCC")
    ax.spines["top"].set_visible(False)
    ax.spines["right"].set_visible(False)
    ax.spines["left"].set_color("#88CCDD")
    ax.spines["bottom"].set_color("#88CCDD")

    ax.margins(x=0.01)

    # x 轴标签：点数多时稀疏显示，避免挤在一起
    if x_axis == "time":
        # 真实时间轴：交给 AutoDateLocator 控制刻度密度
        try:
            ax.xaxis.set_major_locator(mdates.AutoDateLocator(minticks=6, maxticks=10))
            ax.xaxis.set_major_formatter(mdates.DateFormatter("%m-%d"))
        except Exception:
            # fallback：仍然手动稀疏刻度
            if len(labels) > 16:
                step = max(1, len(labels) // 10)
                ax.set_xticks(x_values[::step])
                ax.set_xticklabels(labels[::step], rotation=35, ha="right")
            else:
                ax.set_xticks(x_values)
                ax.set_xticklabels(labels, rotation=35, ha="right")
    else:
        if len(labels) > 16:
            step = max(1, len(labels) // 10)
            ax.set_xticks(x_values[::step])
            ax.set_xticklabels(labels[::step], rotation=35, ha="right")
        else:
            ax.set_xticks(x_values)
            ax.set_xticklabels(labels, rotation=35, ha="right")

    ax.tick_params(axis="x", labelsize=11, colors="#444444")
    ax.tick_params(axis="y", labelsize=11, colors="#444444")

    ax.legend(loc="upper left", frameon=True)

    with TemporaryFile("wb+") as temp_file:
        # 不使用 bbox_inches='tight'，避免输出尺寸被裁剪到不等于背景尺寸
        plt.savefig(temp_file, dpi=dpi, transparent=True, bbox_inches=None, pad_inches=0)
        plt.close()

        temp_file.seek(0)
        plot_img = Image.open(temp_file).convert("RGBA")

        # 添加背景
        if os.path.exists(bg_path):
            bg_img = Image.open(bg_path).convert("RGBA")

            # 若 plot 尺寸与背景不一致，则等比缩放 plot 以适配背景。
            if plot_img.size != bg_img.size:
                bw, bh = bg_img.size
                pw, ph = plot_img.size
                scale = min(bw / pw, bh / ph)
                tw = max(1, int(pw * scale))
                th = max(1, int(ph * scale))
                plot_img = plot_img.resize((tw, th))

                canvas = Image.new("RGBA", bg_img.size, (0, 0, 0, 0))
                canvas.alpha_composite(plot_img, ((bw - tw) // 2, (bh - th) // 2))
                plot_img = canvas

            mask = Image.new("RGBA", bg_img.size, (255, 255, 255, 100))
            bg_img = Image.alpha_composite(bg_img, mask)
            final_img = Image.alpha_composite(bg_img, plot_img)
        else:
            final_img = plot_img

    return final_img



import re
from functools import wraps
from pathlib import Path
from random import choice
from time import perf_counter
from traceback import format_exc
from typing import Literal

from aiohttp.client_exceptions import ClientResponseError
from arclet.alconna import Alconna, AllParam, Args
from maimai_py import LXNSProvider
from nonebot import get_driver, logger
from nonebot.adapters import Event
from nonebot.exception import FinishedException
from nonebot.params import Depends
from nonebot.rule import to_me
from nonebot_plugin_alconna import (
    AlconnaMatch,
    At,
    CommandMeta,
    Match,
    Subcommand,
    UniMessage,
    on_alconna,
)
from nonebot_plugin_alconna.uniseg import Image as UniImage
from nonebot_plugin_orm import async_scoped_session

from .config import config
from .constants import _MAI_VERSION_MAP
from .database import MaiPlayCountORM, MaiSongAliasORM, MaiSongORM, UserBindInfoORM
from .extra_proxy import (
    get_maistatus,
    run_divingfish_import_workflow,
    run_extend_score_workflow,
    run_extend_ticket_workflow,
    run_extent_force_logout,
    run_unlock_workflow,
)
from .functions.analysis import get_player_strength
from .functions.fortunate import generate_today_fortune
from .functions.maistatus import capture_maimai_status_png
from .functions.n50 import get_players_n50
from .functions.process import (
    ProcessDataError,
    get_level_process_data,
    get_plate_process_data,
)
from .functions.recommend_songs import get_player_raise_score_songs
from .functions.song_tags import SONG_TAGS_DATA_AVAILABLE, get_songs_tags
from .models.song import MaiSong
from .painters import (
    DrawScores,
    draw_player_rating_trend,
    draw_player_strength_analysis,
    image_to_bytes,
)
from .painters.process import ProcessPainter
from .renderer import PicRenderer
from .score import (
    DivingFishScoreProvider,
    LXNSScoreProvider,
    MaimaiPyScoreProvider,
    get_divingfish_provider,
    get_lxns_provider,
    get_maimaipy_provider,
)
from .score.providers.maimai import MaimaiPyParams
from .updater.songs import update_song_alias_list
from .utils import get_song_by_id_or_alias, is_float

renderer = PicRenderer()


def _build_song_info_message(user_id: str, song: MaiSong) -> UniMessage:
    """Reuseable builder for song info replies."""

    song_cover = Path(config.static_resource_path) / "mai" / "cover" / f"{song.id}.png"

    if not song_cover.exists():
        dx_song_id = song.id + 10000  # DX 版封面
        song_cover = Path(config.static_resource_path) / "mai" / "cover" / f"{dx_song_id}.png"
        if not song_cover.exists():
            logger.warning(f"未找到乐曲 {song.id} 的封面图片")
            song_cover = Path(config.static_resource_path) / "mai" / "cover" / "0.png"

    response_difficulties_content = []

    if song.difficulties.standard:
        std_diffs = "/".join([str(diff.level_value) for diff in song.difficulties.standard])
        response_difficulties_content.append(f"定数: {std_diffs}")

        if song.difficulties.standard[0].level_fit is not None:
            fit_diffs = "/".join(
                [
                    "{:.2f}".format(diff.level_fit) if diff.level_fit is not None else f"{diff.level}"
                    for diff in song.difficulties.standard
                ]
            )
            response_difficulties_content.append(f"拟合定数: {fit_diffs}")

        if SONG_TAGS_DATA_AVAILABLE:
            song_tags_content = []
            tags_expert = get_songs_tags(song.title, "std", "expert")
            tags_master = get_songs_tags(song.title, "std", "master")
            if tags_expert:
                song_tags_content.append(f"{', '.join(tags_expert)}(Expert)")
            if tags_master:
                song_tags_content.append(f"{', '.join(tags_master)}(Master)")
            if len(song.difficulties.standard) == 5:
                tags_remaster = get_songs_tags(song.title, "std", "remaster")
                song_tags_content.append(f"{', '.join(tags_remaster)}(Re:master)" if tags_remaster else "")
            if song_tags_content:
                response_difficulties_content.append("铺面标签: " + "; ".join(song_tags_content))

    if song.difficulties.dx:
        dx_diffs = "/".join([str(diff.level_value) for diff in song.difficulties.dx])
        response_difficulties_content.append(f"定数(DX): {dx_diffs}")

        if song.difficulties.dx[0].level_fit is not None:
            fit_diffs = "/".join(
                [
                    "{:.2f}".format(diff.level_fit) if diff.level_fit is not None else f"{diff.level}"
                    for diff in song.difficulties.dx
                ]
            )
            response_difficulties_content.append(f"拟合定数(DX): {fit_diffs}")

        if SONG_TAGS_DATA_AVAILABLE:
            song_tags_content = []
            tags_expert = get_songs_tags(song.title, "dx", "expert")
            tags_master = get_songs_tags(song.title, "dx", "master")
            if tags_expert:
                song_tags_content.append(f"{', '.join(tags_expert)}(Expert)")
            if tags_master:
                song_tags_content.append(f"{', '.join(tags_master)}(Master)")
            if len(song.difficulties.dx) == 5:
                tags_remaster = get_songs_tags(song.title, "dx", "remaster")
                song_tags_content.append(f"{', '.join(tags_remaster)}(Re:master)" if tags_remaster else "")
            if song_tags_content:
                response_difficulties_content.append("铺面标签(DX): " + "; ".join(song_tags_content))

    return UniMessage(
        [
            At(flag="user", target=user_id),
            UniImage(path=song_cover),
            _MAI_SONG_INFO_TEMPLATE.format(
                title=song.title,
                id=song.id,
                artist=song.artist,
                genre=song.genre,
                bpm=song.bpm,
                version=_MAI_VERSION_MAP.get(song.version // 100, "未知版本"),
            ),
            "\n".join(response_difficulties_content),
        ]
    )


def catch_exception(reply_prefix: str = "发生了未知错误", reply_error: bool = True):
    def decorator(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            try:
                return await func(*args, **kwargs)
            except FinishedException:
                raise
            except Exception as exc:
                logger.error(format_exc())
                reply_message = f"{reply_prefix}: {str(exc)}" if reply_error else reply_prefix
                await UniMessage(reply_message).finish()

        return wrapper

    return decorator


COMMAND_PREFIXES = [".", "/"]

_MAI_SONG_INFO_TEMPLATE = """[舞萌DX] 乐曲信息
{title}({id})
艺术家: {artist}
分类: {genre}
BPM: {bpm}
版本: {version}
"""

_DIFFICULTY_VALUE_MAP = {"BASIC": 0, "ADVANCED": 1, "EXPERT": 2, "MASTER": 3, "REMASTER": 4, "RE:MASTER": 4}

alconna_help = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "help",
        meta=CommandMeta("显示帮助信息", usage=".help"),
    ),
    priority=50,
    block=False,
    rule=to_me(),
)

alconna_bind = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "bind",
        Subcommand("help"),
        Subcommand("lxns", Args["token", str], help_text=".bind lxns <落雪咖啡屋的个人 API 密钥> 绑定落雪咖啡屋查分器"),
        Subcommand(
            "divingfish", Args["token", str], help_text=".bind divingfish <水鱼查分器的成绩导入密钥> 绑定水鱼查分器"
        ),
        meta=CommandMeta("[查分器相关]绑定游戏账号/查分器"),
    ),
    priority=10,
    block=True,
    skip_for_unmatch=False,
    rule=to_me(),
)

alconna_import = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "import",
        Subcommand("divingfish", help_text="导入成绩到水鱼查分器"),
        Args["qr_code", str],
        meta=CommandMeta("[舞萌DX]导入游玩次数或同步成绩到查分器", usage=".import [divingfish] <qr_code>"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_ticket = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "ticket",
        Args["qr_code", str],
        meta=CommandMeta("[舞萌DX]发送六倍票", usage=".ticket <qr_code>"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_logout = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "logout",
        Args["qr_code", str],
        meta=CommandMeta("[舞萌DX]尝试强制登出", usage=".logout <qr_code>"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_unlock = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "unlock",
        Args["qr_code", str],
        meta=CommandMeta("[舞萌DX]解锁新框紫铺", usage=".unlock <qr_code>"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_unbind = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "unbind",
        Args["provider", Literal["all", "lxns", "divingfish", "maimai"]],
        meta=CommandMeta("[查分器相关]解绑查分器账号", usage=".unbind <all|lxns|divingfish|maimai>"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_source = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "source",
        Args["provider", str],
        meta=CommandMeta("[查分器相关]设置默认查分器", usage=".source <lxns|divingfish>"),
    ),
    priority=10,
    block=True,
    aliases={"provider"},
    rule=to_me(),
)

alconna_b50 = on_alconna(
    Alconna(COMMAND_PREFIXES, "b50", meta=CommandMeta("[舞萌DX]生成玩家 Best 50")),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_ap50 = on_alconna(
    Alconna(COMMAND_PREFIXES, "ap50", meta=CommandMeta("[舞萌DX]生成玩家 ALL PERFECT 50")),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_r50 = on_alconna(
    Alconna(COMMAND_PREFIXES, "r50", meta=CommandMeta("[舞萌DX]生成玩家 Recent 50 (需绑定落雪查分器)")),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_pc50 = on_alconna(
    Alconna(
        COMMAND_PREFIXES, "pc50", meta=CommandMeta("[舞萌DX]生成玩家游玩次数 Top-50 (需使用`.import`导入游玩次数)")
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_n50 = on_alconna(
    Alconna(COMMAND_PREFIXES, "n50", meta=CommandMeta("[舞萌DX]生成玩家基于拟合系数的 Top-50")),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_minfo = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "minfo",
        Args["name", AllParam(str)],
        meta=CommandMeta("[舞萌DX]获取乐曲信息", usage=".minfo <id|别名>"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_random = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "random",
        Args["filters", AllParam(str)],
        meta=CommandMeta("[舞萌DX]随机抽取乐曲", usage=".random [难度] [等级|定数]"),
    ),
    aliases={"随机乐曲", "随"},
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_alias = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "alias",
        Subcommand("help"),
        Subcommand(
            "add", Args["song_id", int], Args["alias", str], help_text=".alias add <乐曲ID> <别名> 添加乐曲别名"
        ),
        Subcommand("update", help_text=".alias update 更新本地乐曲别名列表"),
        Subcommand("query", Args["name", AllParam(str)], help_text=".alias query <id|别名> 查询该歌曲有什么别名"),
        meta=CommandMeta("[舞萌DX]乐曲别名管理"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_score = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "score",
        Args["name", AllParam(str)],
        meta=CommandMeta("[舞萌DX]获取单曲游玩情况", usage=".score <id|别名>"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_scorelist = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "scorelist",
        Args["arg", str],
        meta=CommandMeta(
            "[舞萌DX]获取指定条件的成绩列表", usage=".scorelist <level|ach|diff>", example=".scorelist ach100.4"
        ),
    ),
    aliases={"scoreslist"},
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_plate_process = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        r"re:([真超檄橙暁晓桃櫻樱紫菫堇白雪輝辉舞霸熊華华爽煌星宙祭祝双宴镜彩])([極极将舞神者]舞?)进度\s?(.+)?",
        meta=CommandMeta(
            "[舞萌DX]牌子进度",
            usage=".真极进度 / .熊将进度 难 / .紫舞舞进度",
        ),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_level_process = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        r"re:([0-9]+\+?)\s?([abcdsfxp\+]+)\s?([\u4e00-\u9fa5]+)?进度\s?([0-9]+)?\s?(.+)?",
        meta=CommandMeta(
            "[舞萌DX]等级进度",
            usage=".13+ sss 进度 / .14 ap 进度",
        ),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_update = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "update",
        Subcommand("songs", help_text=".update songs 更新乐曲信息数据库"),
        Subcommand("alias", help_text=".update alias 更新乐曲别名列表"),
        Subcommand("chart", help_text=".update chart 更新 music_chart.json 文件"),
        meta=CommandMeta("[舞萌DX]更新乐曲信息或别名列表"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_fortune = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "fortune",
        meta=CommandMeta("[舞萌DX]获取今日上机运势"),
    ),
    aliases={"今日舞萌"},
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_analysis = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "analysis",
        meta=CommandMeta("[舞萌DX]根据 B100 获取玩家成分"),
    ),
    aliases={"底力分析", "成分分析"},
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_trend = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "trend",
        meta=CommandMeta("[舞萌DX]生成玩家 DX Rating 趋势（需绑定落雪查分器）"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_recommend = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "recommend",
        meta=CommandMeta("[舞萌DX]生成随机推分曲目"),
    ),
    aliases={"什么推分", "推分", "推分推荐"},
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_maistatus = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "maistatus",
        meta=CommandMeta("[舞萌DX]检测服务器状态（可选渲染状态页截图）", usage=".maistatus"),
    ),
    aliases={"舞萌状态"},
    priority=10,
    block=True,
    rule=to_me(),
)

alconna_rikka = on_alconna(
    Alconna(
        COMMAND_PREFIXES,
        "rikka",
        meta=CommandMeta("输出 Rikka 插件信息"),
    ),
    priority=10,
    block=True,
    rule=to_me(),
)


@alconna_help.handle()
async def handle_help(event: Event):
    user_id = event.get_user_id()

    help_text = (
        "Rikka 查分器帮助:\n"
        ".bind <查分器名称> <API密钥> 绑定查分器账号\n"
        ".unbind <查分器名称> 解绑游戏账号/查分器\n"
        ".source <查分器名称> 设置默认查分器\n"
        ".b50 获取玩家 Best 50\n"
        ".ap50 获取玩家 ALL PERFECT 50\n"
        ".r50 获取玩家 Recent 50 (需绑定落雪查分器)\n"
        ".n50 获取玩家拟合系数 Top-50\n"
        f".pc50 生成玩家游玩次数 Top50 （{'当前不可用' if not config.enable_arcade_provider else '需通过 `.import` 导入游戏成绩'})\n"
        ".random 随机获取一首乐曲（可选难度、等级、定数）\n"
        ".minfo <乐曲ID/别名> 获取乐曲信息\n"
        ".alias 管理乐曲别名（添加、查询、更新）\n"
        ".score <乐曲ID/别名> 获取单曲游玩情况\n"
        ".scorelist <level|ach|diff> 获取指定条件的成绩列表\n"
        ".update songs 更新乐曲信息数据库\n"
        ".update alias 更新乐曲别名列表\n"
        ".trend 获取玩家的 DX Rating 趋势 （需绑定落雪查分器）\n"
        f".成分分析 根据 B100 获取玩家成分分析 {'(当前不可用)' if not SONG_TAGS_DATA_AVAILABLE else ''}\n"
        ".今日舞萌 获取今日出勤运势\n"
        ".舞萌状态 检测服务器状态\n"
        f".推分推荐 生成随机推分曲目 {'(当前不可用)' if not SONG_TAGS_DATA_AVAILABLE else ''}\n"
        f".import [divingfish] <qr_code> 导入游玩次数或同步到水鱼 {'当前不可用' if not config.enable_arcade_provider else ''}\n"
        f".ticket <qr_code> 发送六倍票 {'当前不可用' if not config.enable_arcade_provider else ''}\n"
        f".logout <qr_code> 尝试强制登出 {'当前不可用' if not config.enable_arcade_provider else ''}\n"
        f".unlock <qr_code> 解锁新框紫铺 {'当前不可用' if not config.enable_arcade_provider else ''}\n"
    )

    await UniMessage(
        [
            At(flag="user", target=user_id),
            help_text,
        ]
    ).finish()


@alconna_bind.assign("lxns")
@catch_exception("尝试连接到落雪服务器时遇到错误")
async def handle_bind_lxns(
    event: Event,
    db_session: async_scoped_session,
    score_provider: LXNSScoreProvider = Depends(get_lxns_provider),
    token: Match[str] = AlconnaMatch("token"),
):
    user_id = event.get_user_id()

    if not token.available:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请输入有效的落雪咖啡屋 API 密钥",
            ]
        ).finish()
        return

    lxns_token = token.result

    logger.debug("尝试验证玩家 API 密钥是否正确")
    try:
        player_info = await score_provider.fetch_player_info_by_user_token(lxns_token)
    except ClientResponseError as e:
        logger.warning(f"玩家提供的 API 可能有误: {e.message}")
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请输入有效的落雪咖啡屋 API 密钥",
            ]
        ).finish()
        return

    await UserBindInfoORM.set_user_friend_code(db_session, user_id, player_info.friend_code)  # type: ignore
    await UserBindInfoORM.set_lxns_api_key(db_session, user_id, lxns_token)

    await UniMessage(
        [
            At(flag="user", target=user_id),
            f"已绑定至游戏账号: {player_info.name} ⭐",
        ]
    ).finish()


@alconna_bind.assign("divingfish")
async def handle_bind_divingfish(
    event: Event,
    db_session: async_scoped_session,
    score_provider: DivingFishScoreProvider = Depends(get_divingfish_provider),
    token: Match[str] = AlconnaMatch("token"),
):
    user_id = event.get_user_id()

    if not token.available:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请输入有效的水鱼查分器成绩导入密钥",
            ]
        ).finish()
        return

    import_token = token.result

    logger.debug("尝试验证玩家 API 密钥是否正确")
    try:
        player_info = await score_provider.fetch_player_records_by_import_token(import_token)
    except ClientResponseError as e:
        logger.warning(f"玩家提供的 import_token 可能有误: {e.message}")
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请输入有效的水鱼查分器成绩导入密钥",
            ]
        ).finish()
        return

    await UserBindInfoORM.set_diving_fish_import_token(db_session, user_id, import_token, player_info["username"])

    await UniMessage(
        [
            At(flag="user", target=user_id),
            f"已绑定至水鱼账号: {player_info['username']} ⭐",
        ]
    ).finish()


@alconna_bind.assign("help")
async def handle_bind_help(event: Event):
    user_id = event.get_user_id()

    help_text = (
        "查分器绑定帮助:\n"
        ".bind lxns <落雪咖啡屋的个人 API 密钥> 绑定落雪咖啡屋查分器\n"
        ".bind divingfish <水鱼查分器的成绩导入密钥> 绑定水鱼查分器\n"
    )

    await UniMessage(
        [
            At(flag="user", target=user_id),
            help_text,
        ]
    ).finish()


@alconna_bind.assign("$main")
async def handle_bind_main(event: Event, db_session: async_scoped_session):
    user_id = event.get_user_id()
    user_bind_info = await UserBindInfoORM.get_user_bind_info(db_session, user_id)
    lxns_binded = False
    divingfish_binded = False
    default_source = "未设置"
    if user_bind_info:
        if user_bind_info.lxns_api_key and user_bind_info.friend_code:
            lxns_binded = True
        if user_bind_info.diving_fish_import_token:
            divingfish_binded = True
        if user_bind_info.default_provider:
            default_source = "落雪" if user_bind_info.default_provider == "lxns" else "水鱼"

    lxns = "已绑定" if lxns_binded else "未绑定"
    divingfish = "已绑定" if divingfish_binded else "未绑定"
    user_bind_detail = (
        f"查分器绑定情况:\n- 落雪查分器: {lxns}\n - 水鱼查分器: {divingfish}\n\n当前默认查分源: {default_source}"
    )

    await UniMessage(
        [
            At(flag="user", target=user_id),
            user_bind_detail,
        ]
    ).send()

    return await handle_bind_help(event)


@alconna_import.assign("$main")
@catch_exception("导入游玩次数失败")
async def handle_import_play_count(
    event: Event,
    db_session: async_scoped_session,
    qr_code: Match[str] = AlconnaMatch("qr_code"),
):
    user_id = event.get_user_id()

    if not config.enable_arcade_provider:
        await UniMessage([At(flag="user", target=user_id), "机台源不可用，无法执行该操作"]).finish()
        return

    if not qr_code.available or not qr_code.result:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请提供二维码内容: .import <qr_code>",
            ]
        ).finish()
        return

    workflow_result = await run_extend_score_workflow(qr_code.result)

    records: list[tuple[int, int, int]] = []
    for item in workflow_result:
        if not isinstance(item, dict):
            continue
        try:
            song_id = int(item["musicId"])
            difficulty = int(item["level"])
            play_count = int(item["playCount"])
        except (KeyError, TypeError, ValueError):
            continue
        records.append((song_id, difficulty, play_count))

    if not records:
        await UniMessage([At(flag="user", target=user_id), "未获取到可用的游玩次数数据"]).finish()
        return

    imported = await MaiPlayCountORM.upsert_user_play_counts(db_session, user_id, records)

    await UniMessage(
        [
            At(flag="user", target=user_id),
            f"已导入 {imported} 条游玩次数记录",
        ]
    ).finish()


@alconna_import.assign("divingfish")
@catch_exception("更新水鱼查分器失败")
async def handle_import_divingfish(
    event: Event, db_session: async_scoped_session, qr_code: Match[str] = AlconnaMatch("qr_code")
):
    user_id = event.get_user_id()

    if not config.enable_arcade_provider:
        await UniMessage([At(flag="user", target=user_id), "机台源不可用，无法执行该操作"]).finish()
        return

    if not qr_code.available or not qr_code.result:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请提供二维码内容: .import <qr_code>",
            ]
        ).finish()
        return

    # 检查是否绑定了水鱼 Token
    bind_info = await UserBindInfoORM.get_user_bind_info(db_session, user_id)
    if not bind_info or not bind_info.diving_fish_import_token:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "未绑定水鱼查分器导入 Token，请使用 .bind divingfish <Token> 命令绑定",
            ]
        ).finish()
        return  # 防止 mypy 报错 bind_info 可能为 None 的情况

    import_token = bind_info.diving_fish_import_token

    await run_divingfish_import_workflow(qr_code.result, import_token)

    await UniMessage([At(flag="user", target=user_id), "水鱼查分器更新成功！"]).finish()


@alconna_ticket.handle()
@catch_exception("发票失败")
async def handle_ticket(
    event: Event,
    qr_code: Match[str] = AlconnaMatch("qr_code"),
):
    user_id = event.get_user_id()

    if not qr_code.available or not qr_code.result:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请提供二维码内容: .ticket <qr_code>",
            ]
        ).finish()
        return

    await run_extend_ticket_workflow(qr_code.result)

    await UniMessage([At(flag="user", target=user_id), "已成功发送了 6 倍票"]).finish()


@alconna_logout.handle()
@catch_exception(reply_prefix="强制登出失败")
async def handle_logout(
    event: Event,
    qr_code: Match[str] = AlconnaMatch("qr_code"),
):
    user_id = event.get_user_id()

    if not qr_code.available or not qr_code.result:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请提供二维码内容: .logout <qr_code>",
            ]
        ).finish()
        return

    await run_extent_force_logout(qr_code.result)
    await UniMessage([At(flag="user", target=user_id), "已尝试强制登出，请尝试重新登录"]).finish()


@alconna_unlock.handle()
@catch_exception()
async def handle_rikka_unlock(
    event: Event,
    qr_code: Match[str] = AlconnaMatch("qr_code"),
):
    user_id = event.get_user_id()

    if not config.enable_arcade_provider:
        await UniMessage([At(flag="user", target=user_id), "机台源不可用，无法执行该操作"]).finish()

    if not qr_code.available or not qr_code.result:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请提供二维码内容: .unlock <qr_code>",
            ]
        ).finish()
        return

    await run_unlock_workflow(qr_code.result)
    await UniMessage([At(flag="user", target=user_id), "解锁成功！"]).finish()


@alconna_unbind.handle()
async def handle_unbind(
    event: Event,
    db_session: async_scoped_session,
    provider: Match[Literal["all", "lxns", "divingfish", "maimai"]] = AlconnaMatch("provider"),
):
    user_id = event.get_user_id()

    if not provider.available or provider.result not in ["all", "lxns", "divingfish", "maimai"]:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请输入有效的查分器名称: all, lxns, divingfish 或 maimai",
            ]
        ).finish()
        return

    provider_name = provider.result if provider.result != "all" else None

    try:
        await UserBindInfoORM.unset_user_bind_info(db_session, user_id, provider_name)
    except ValueError as e:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                str(e),
            ]
        ).finish()
        return

    await UniMessage(
        [
            At(flag="user", target=user_id),
            f"已解绑查分器: {provider.result}",
        ]
    ).finish()


@alconna_source.handle()
async def handle_source(
    event: Event,
    db_session: async_scoped_session,
    provider: Match[str] = AlconnaMatch("provider"),
):
    user_id = event.get_user_id()

    if not provider.available or provider.result not in ["lxns", "divingfish"]:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "请输入有效的查分器名称: lxns 或 divingfish",
            ]
        ).finish()
        return

    try:
        await UserBindInfoORM.set_default_provider(db_session, user_id, provider.result)  # type: ignore
    except ValueError as e:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                str(e),
            ]
        ).finish()
        return

    await UniMessage(
        [
            At(flag="user", target=user_id),
            f"已将默认查分器设置为: {provider.result} ⭐",
        ]
    ).finish()


@alconna_b50.handle()
@catch_exception()
async def handle_mai_b50(
    event: Event,
    db_session: async_scoped_session,
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()
    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)

    logger.info(f"[{user_id}] 获取玩家 Best50, 查分器类型: {type(provider)}")
    logger.debug(f"[{user_id}] 1/4 获得用户鉴权凭证...")

    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(db_session, user_id, provider)
    params = score_provider.ParamsType(provider, identifier)

    logger.debug(f"[{user_id}] 2/4 发起 API 请求玩家信息...")
    player_info = await score_provider.fetch_player_info(params)

    logger.debug(f"[{user_id}] 3/4 发起 API 请求玩家 Best50...")
    player_b50 = await score_provider.fetch_player_b50(params)

    logger.debug(f"[{user_id}] 4/4 渲染玩家数据...")
    pic = await renderer.render_mai_player_best50(player_b50, player_info)

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=pic)]).finish()


@alconna_ap50.handle()
@catch_exception()
async def handle_mai_ap50(
    event: Event,
    db_session: async_scoped_session,
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()
    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)

    logger.info(f"[{user_id}] 获取玩家 AP50, 查分器类型: {type(score_provider)}")
    logger.debug(f"[{user_id}] 1/4 获得用户鉴权凭证...")

    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(db_session, user_id, provider)
    params = score_provider.ParamsType(provider, identifier)

    logger.debug(f"[{user_id}] 2/4 发起 API 请求玩家信息...")
    player_info = await score_provider.fetch_player_info(params)

    logger.debug(f"[{user_id}] 3/4 发起 API 请求玩家 AP 50...")
    if isinstance(provider, LXNSProvider):
        user_bind_info = await UserBindInfoORM.get_user_bind_info(db_session, user_id)
        if user_bind_info is None or user_bind_info.lxns_api_key is None:
            await UniMessage("你还没有绑定任何查分器喵，请先用 /bind 绑定一个查分器谢谢喵").finish()
            return  # Avoid TypeError.

        identifier.credentials = user_bind_info.lxns_api_key
        params = score_provider.ParamsType(provider, identifier)

    player_ap50 = await score_provider.fetch_player_ap50(params)

    logger.debug(f"[{user_id}] 4/4 渲染玩家数据...")
    pic = await renderer.render_mai_player_best50(player_ap50, player_info)

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=pic)]).finish()


@alconna_r50.handle()
@catch_exception()
async def handle_mai_r50(
    event: Event, db_session: async_scoped_session, score_provider: LXNSScoreProvider = Depends(get_lxns_provider)
):
    user_id = event.get_user_id()

    logger.info(f"[{user_id}] 获取玩家 Recent 50, 查分器名称: {score_provider.provider}")
    logger.debug(f"[{user_id}] 1/4 尝试从数据库中获取玩家绑定信息...")

    user_bind_info = await UserBindInfoORM.get_user_bind_info(db_session, user_id)

    new_player_friend_code = None
    if user_bind_info is None:
        logger.warning(f"[{user_id}] 未能获取玩家码，数据库中不存在绑定的玩家数据")
        logger.debug(f"[{user_id}] 1/4 尝试通过 QQ 请求玩家数据")
        try:
            player_info = await score_provider.fetch_player_info_by_qq(user_id)
            new_player_friend_code = player_info.friend_code
        except ClientResponseError as e:
            logger.warning(f"[{user_id}] 无法通过 QQ 号请求玩家数据: {e.code}: {e.message}")

            await UniMessage(
                [
                    At(flag="user", target=user_id),
                    "查询 Recent 50 的操作需要绑定落雪查分器喵，还请使用 /bind 指令进行绑定喵呜",
                ]
            ).finish()

            return

    friend_code = new_player_friend_code or user_bind_info.friend_code  # type: ignore
    if not friend_code:
        logger.warning(f"[{user_id}] 无法获取好友码，无法继续查询。")
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "无法获取好友码，请确认已绑定或查分器可用。",
            ]
        ).finish()
        return
    logger.debug(f"[{user_id}] 2/4 发起 API 请求玩家信息...")
    params = score_provider.ParamsType(friend_code=friend_code)
    player_info = await score_provider.fetch_player_info(params)

    logger.debug(f"[{user_id}] 3/4 发起 API 请求玩家 Recent 50...")
    player_r50 = await score_provider.fetch_player_r50(friend_code)

    logger.debug(f"[{user_id}] 4/4 渲染玩家数据...")
    pic = await renderer.render_mai_player_scores(player_r50, player_info, title="Recent 50")

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=pic)]).finish()


@alconna_pc50.handle()
@catch_exception()
async def handle_pc50(
    event: Event,
    db_session: async_scoped_session,
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()
    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)

    if not config.enable_arcade_provider:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "管理员未启用pc数查询喵呜",
            ]
        ).finish()

    logger.info(f"[{user_id}] 获取玩家 PC50, 查分器类型: {type(provider)}")

    logger.debug(f"[{user_id}] 1/4 获得用户鉴权凭证...")

    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(db_session, user_id, provider)
    params = score_provider.ParamsType(provider, identifier)

    logger.debug(f"[{user_id}] 2/4 发起 API 请求玩家信息...")
    player_info = await score_provider.fetch_player_info(params)

    logger.debug(f"[{user_id}] 3/4 发起 API 请求玩家所有成绩...")
    if isinstance(provider, LXNSProvider):
        user_bind_info = await UserBindInfoORM.get_user_bind_info(db_session, user_id)
        if user_bind_info is None or user_bind_info.lxns_api_key is None:
            await UniMessage("你还没有绑定任何查分器喵，请先用 /bind 绑定一个查分器谢谢喵").finish()
            return  # Avoid TypeError.

        identifier.credentials = user_bind_info.lxns_api_key
        params = score_provider.ParamsType(provider, identifier)

    player_scores = await score_provider.fetch_player_pc50(params)

    logger.debug(f"[{user_id}] 4/4 渲染玩家数据...")
    pic = await renderer.render_mai_player_best50(player_scores, player_info)

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=pic)]).finish()


@alconna_n50.handle()
@catch_exception()
async def handle_n50(
    event: Event,
    db_session: async_scoped_session,
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()
    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)

    logger.info(f"[{user_id}] 获取玩家 N50, 查分器类型: {type(provider)}")
    logger.debug(f"[{user_id}] 1/5 获得用户鉴权凭证...")

    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(db_session, user_id, provider)
    params = score_provider.ParamsType(provider, identifier)

    logger.debug(f"[{user_id}] 2/5 发起 API 请求玩家信息...")
    player_info = await score_provider.fetch_player_info(params)

    logger.debug(f"[{user_id}] 3/5 发起 API 请求玩家所有成绩")
    if isinstance(provider, LXNSProvider):
        user_bind_info = await UserBindInfoORM.get_user_bind_info(db_session, user_id)
        if user_bind_info is None or user_bind_info.lxns_api_key is None:
            await UniMessage("你还没有绑定任何查分器喵，请先用 /bind 绑定一个查分器谢谢喵").finish()
            return  # Avoid TypeError.

        identifier.credentials = user_bind_info.lxns_api_key
        params = score_provider.ParamsType(provider, identifier)
    player_scores = await score_provider.fetch_player_scoreslist(params)

    logger.debug(f"[{user_id}] 4/5 计算 N50...")
    player_n50 = get_players_n50(player_scores)
    player_info.rating = player_n50.rating

    logger.debug(f"[{user_id}] 5/5 渲染玩家数据...")
    pic = await renderer.render_mai_player_best50(player_n50, player_info, calc_song_level_value=False)

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=pic)]).finish()


@alconna_minfo.handle()
@catch_exception()
async def handle_minfo(
    event: Event,
    db_session: async_scoped_session,
    name: Match[UniMessage] = AlconnaMatch("name"),
):
    user_id = event.get_user_id()

    if not name.available:
        await UniMessage([At(flag="user", target=user_id), "请输入有效的乐曲ID/名称/别名！"]).finish()

    raw_query = name.result.extract_plain_text()
    logger.info(f"[{user_id}] 查询乐曲信息, 查询内容: {raw_query}")

    logger.debug(f"[{user_id}] 1/4 通过乐曲ID/别名查询乐曲信息...")
    try:
        song = await get_song_by_id_or_alias(db_session, raw_query)
    except ValueError as e:
        await UniMessage([At(flag="user", target=user_id), str(e)]).finish()
        return

    logger.debug(f"[{user_id}] 2/2 构建乐曲信息模板...")
    response_content = _build_song_info_message(user_id, song)

    await response_content.finish()


@alconna_random.handle()
@catch_exception()
async def handle_random(
    event: Event,
    db_session: async_scoped_session,
    filters: Match[UniMessage] = AlconnaMatch("filters"),
):
    user_id = event.get_user_id()

    raw_filters = filters.result.extract_plain_text().strip() if filters.available else ""
    tokens = [tok for tok in raw_filters.split() if tok]

    if len(tokens) > 2:
        await UniMessage([At(flag="user", target=user_id), "参数过多，最多填写难度与等级/定数两个条件哦~"]).finish()
        return

    diff_value = None
    diff_name = None
    level_value = None
    level_const = None
    invalid_tokens: list[str] = []

    for tok in tokens:
        up_tok = tok.upper()
        up_tok = "REMASTER" if up_tok == "RE:MASTER" else up_tok
        if up_tok in _DIFFICULTY_VALUE_MAP:
            if diff_value is not None:
                invalid_tokens.append(tok)
                continue
            diff_value = _DIFFICULTY_VALUE_MAP[up_tok]
            diff_name = up_tok
            continue

        if re.fullmatch(r"\d+\+?", tok):
            if level_value is not None:
                invalid_tokens.append(tok)
                continue
            level_value = tok
            continue

        if "." in tok and re.fullmatch(r"\d+(?:\.\d+)?", tok):
            if level_const is not None:
                invalid_tokens.append(tok)
                continue
            try:
                level_const = float(tok)
            except ValueError:
                invalid_tokens.append(tok)
            continue

        invalid_tokens.append(tok)

    if invalid_tokens:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                f"以下参数无法识别: {' '.join(invalid_tokens)}，请使用难度(BASIC~RE:MASTER)、等级(如12+)或定数(如12.7)重试~",
            ]
        ).finish()
        return

    logger.info(f"[{user_id}] 随机抽取乐曲, 条件 diff={diff_name}, level={level_value}, const={level_const}")

    song_ids = await MaiSongORM.get_all_song_ids(db_session)
    songs = await MaiSongORM.get_songs_info_by_ids(db_session, list(song_ids))

    def match_song(song: MaiSong) -> bool:
        candidates = list(song.difficulties.standard) + list(song.difficulties.dx)
        for d in candidates:
            if diff_value is not None and d.difficulty != diff_value:
                continue
            if level_value and d.level != level_value:
                continue
            if level_const is not None and abs(d.level_value - level_const) > 1e-6:
                continue
            return True
        return False

    filtered_songs = [s for s in songs if match_song(s)]

    if not filtered_songs:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "未找到符合条件的乐曲喵，试试放宽条件再来一次吧~",
            ]
        ).finish()
        return

    song = choice(filtered_songs)
    response = _build_song_info_message(user_id, song)

    await response.finish()


@alconna_alias.assign("update")
async def handle_alias_update(
    event: Event,
    db_session: async_scoped_session,
):
    user_id = event.get_user_id()
    nb_config = get_driver().config

    if user_id not in nb_config.superusers:
        await UniMessage("更新乐曲别名需要管理员权限哦").finish()

    logger.info(f"[{user_id}] 更新乐曲别名列表")

    await update_song_alias_list(db_session)

    logger.info(f"[{user_id}] 乐曲别名列表更新完成")

    await UniMessage(
        [
            At(flag="user", target=user_id),
            "乐曲别名列表已更新完成 ⭐",
        ]
    ).finish()


@alconna_alias.assign("add")
async def handle_alias_add(
    event: Event,
    db_session: async_scoped_session,
    name: Match[str] = AlconnaMatch("alias"),
    song_id: Match[int] = AlconnaMatch("song_id"),
):
    user_id = event.get_user_id()

    if config.add_alias_need_admin:
        nb_config = get_driver().config

        if user_id not in nb_config.superusers:
            await UniMessage("更新乐曲别名需要管理员权限哦").finish()

    logger.info(f"[{user_id}] 添加乐曲别名, 乐曲ID: {song_id.result}, 别名: {name.result}")

    await MaiSongAliasORM.add_custom_alias(db_session, song_id.result, name.result)

    await UniMessage(
        [
            At(flag="user", target=user_id),
            f"已为乐曲 ID {song_id.result} 添加别名: {name.result} ⭐",
        ]
    ).finish()


@alconna_alias.assign("query")
async def handle_alias_query(
    event: Event,
    db_session: async_scoped_session,
    name: Match[UniMessage] = AlconnaMatch("name"),
):
    user_id = event.get_user_id()

    if not name.available:
        await UniMessage([At(flag="user", target=user_id), "请输入有效的乐曲ID/名称/别名！"]).finish()

    raw_query = name.result.extract_plain_text()

    logger.info(f"[{user_id}] 查询乐曲别名, 查询内容: {raw_query}")

    logger.debug(f"[{user_id}] 1/4 通过乐曲ID/别名查询乐曲信息...")
    try:
        song = await get_song_by_id_or_alias(db_session, raw_query)
    except ValueError as e:
        await UniMessage([At(flag="user", target=user_id), str(e)]).finish()
        return

    logger.debug(f"[{user_id}] 2/4 获取乐曲别名列表...")
    aliases = await MaiSongAliasORM.get_aliases(db_session, song.id)

    logger.debug(f"[{user_id}] 3/4 构建乐曲别名模板...")

    if not aliases:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                f"乐曲 ID {song.id} ('{song.title}') 暂无别名记录！",
            ]
        ).finish()
        return

    alias_list_content = ", ".join(aliases)
    await UniMessage(
        [
            At(flag="user", target=user_id),
            f"乐曲 ID {song.id} ('{song.title}') 的别名列表如下：\n{alias_list_content}",
        ]
    ).finish()


@alconna_alias.assign("help")
async def handle_alias_help(event: Event):
    user_id = event.get_user_id()

    help_text = (
        "乐曲别名管理帮助:\n"
        ".alias add <乐曲ID> <别名> 添加乐曲别名\n"
        ".alias update 更新本地乐曲别名列表\n"
        ".alias query <id|别名> 查询该歌曲有什么别名\n"
    )

    await UniMessage(
        [
            At(flag="user", target=user_id),
            help_text,
        ]
    ).finish()


@alconna_alias.assign("$main")
async def handle_alias_main(event: Event):
    return await handle_alias_help(event)


@alconna_score.handle()
@catch_exception()
async def handle_score(
    event: Event,
    db_session: async_scoped_session,
    name: Match[UniMessage] = AlconnaMatch("name"),
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()

    if not name.available:
        await UniMessage([At(flag="user", target=user_id), "请输入有效的乐曲ID/名称/别名！"]).finish()

    raw_query = name.result.extract_plain_text()
    logger.info(f"[{user_id}] 查询单曲游玩情况, 查询内容: {raw_query}")

    logger.debug(f"[{user_id}] 1/5 通过乐曲ID/别名查询乐曲信息...")
    try:
        song = await get_song_by_id_or_alias(db_session, raw_query)
    except ValueError as e:
        await UniMessage([At(flag="user", target=user_id), str(e)]).finish()
        return

    logger.debug(f"[{user_id}] 2/5 推断获取的是 DX 铺面还是标准铺面")
    if raw_query.isdigit():
        if int(raw_query) > 10000 or not song.difficulties.standard:
            is_dx = True
        else:
            is_dx = False
    else:
        is_dx = len(song.difficulties.dx) > 0
    logger.debug(f"[{user_id}] 2/5 推断为 {'DX' if is_dx else '标准'} 铺面")

    logger.debug(f"[{user_id}] 3/5 获得用户鉴权凭证...")
    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)
    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(db_session, user_id, provider)
    params = score_provider.ParamsType(provider, identifier)

    logger.debug(f"[{user_id}] 4/5 发起 API 请求玩家信息...")
    scores = await score_provider.fetch_player_minfo(params, song.id, "dx" if is_dx else "standard")

    if not scores:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                f"未找到乐曲 '{song.title}' 的游玩记录喵~不如先去挑战一下吧~",
            ]
        ).finish()
        return

    logger.debug(f"[{user_id}] 5/5 渲染玩家数据...")
    pic = await renderer.render_mai_player_song_info(song, scores)

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=pic)]).finish()


@alconna_scorelist.handle()
@catch_exception()
async def handle_scorelist(
    event: Event,
    db_session: async_scoped_session,
    arg: Match[str] = AlconnaMatch("arg"),
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()

    if not arg.available or not arg.result:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                (
                    ".scorelist 使用帮助\n"
                    ".scorelist <level> 获取指定等级的成绩列表\n"
                    ".scorelist ach<float> 获取指定达成率的成绩列表\n"
                    ".scorelist <diff> 获取指定铺面难度的成绩列表\n"
                    "eg.\n"
                    ".scorelist 12+\n"
                    ".scorelist ach100.8"
                    ".scorelist expert"
                ),
            ]
        ).finish()

    raw_query = arg.result
    logger.info(f"[{user_id}] 查询指定条件的成绩列表, 查询内容: {raw_query}")
    level = None
    ach = None
    diff = None
    if raw_query.isdigit() or (raw_query.endswith("+") and raw_query[:-1].isdigit()):
        level = raw_query
        title = f"{level} 成绩列表"
    elif raw_query.startswith("ach") and is_float(raw_query[3:]):
        ach = float(raw_query[3:])
        title = f"达成率 {ach} 成绩列表"
    elif raw_query.upper() in ["BASIC", "ADVANCED", "EXPERT", "MASTER", "REMASTER", "RE:MASTER"]:
        diff = raw_query.upper()
        diff = "REMASTER" if diff == "RE:MASTER" else diff
        title = f"铺面等级 {diff} 成绩列表"
    else:
        await UniMessage([At(flag="user", target=user_id), "命令格式错误，请检查后重新输入！"]).finish()
        return

    logger.debug(f"[{user_id}] 1/4 获得用户鉴权凭证...")
    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)
    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(
        db_session, user_id, provider, use_personal_api=True
    )
    params = score_provider.ParamsType(provider, identifier)
    logger.debug(f"[{user_id}] 1/4 鉴权参数: {params}")

    logger.debug(f"[{user_id}] 2/4 发起 API 请求玩家信息...")
    player_info = await score_provider.fetch_player_info(params)

    logger.debug(f"[{user_id}] 3/4 发起 API 请求玩家全部成绩...")
    scores = await score_provider.fetch_player_scoreslist(params, level, ach, diff)  # type: ignore

    if not scores:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "呜呜，未找到符合条件的游玩记录喵",
            ]
        ).finish()
        return

    logger.debug(f"[{user_id}] 4/4 渲染玩家数据...")
    pic = await renderer.render_mai_player_scores(scores[:50], player_info, title)

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=pic)]).finish()


@alconna_plate_process.handle()
@catch_exception()
async def handle_plate_process(
    event: Event,
    db_session: async_scoped_session,
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()

    raw_text = event.get_plaintext().strip()
    for p in COMMAND_PREFIXES:
        if raw_text.startswith(p):
            raw_text = raw_text[len(p) :]
            break

    m = re.match(
        r"^([真超檄橙暁晓桃櫻樱紫菫堇白雪輝辉舞霸熊華华爽煌星宙祭祝双宴镜彩])([極极将舞神者]舞?)进度\s?(.+)?$",
        raw_text,
    )
    if not m:
        await UniMessage([At(flag="user", target=user_id), "命令解析失败，请检查输入格式"]).finish()
        return

    ver_char, plan, extra_text = m.group(1), m.group(2), (m.group(3) or "")
    use_difficult = "难" in extra_text

    if f"{ver_char}{plan}" == "真将":
        await UniMessage([At(flag="user", target=user_id), "真系没有真将哦"]).finish()
        return

    logger.info(f"[{user_id}] 查询牌子进度: {ver_char}{plan} {'难' if use_difficult else ''}")

    logger.debug(f"[{user_id}] 1/3 获得用户鉴权凭证...")
    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)
    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(
        db_session, user_id, provider, use_personal_api=True
    )

    songs = list(MaiSongORM._cache.values())
    if not songs:
        await MaiSongORM.refresh_cache(db_session)
        songs = list(MaiSongORM._cache.values())

    logger.debug(f"[{user_id}] 2/3 发起 API 请求玩家所有成绩")
    scores = await score_provider.fetch_player_scoreslist(
        MaimaiPyParams(score_provider=provider, identifier=identifier)
    )
    try:
        data = get_plate_process_data(songs, scores, ver_char, plan)
    except ProcessDataError as e:
        await UniMessage([At(flag="user", target=user_id), str(e)]).finish()
        return

    logger.debug(f"[{user_id}] 3/3 渲染玩家数据...")
    img = ProcessPainter(data).draw_plate(use_difficult=use_difficult)
    await UniMessage([At(flag="user", target=user_id), UniImage(raw=img.getvalue())]).finish()


@alconna_level_process.handle()
@catch_exception()
async def handle_level_process(
    event: Event,
    db_session: async_scoped_session,
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()

    raw_text = event.get_plaintext().strip()
    for p in COMMAND_PREFIXES:
        if raw_text.startswith(p):
            raw_text = raw_text[len(p) :]
            break

    m = re.match(r"^([0-9]+\+?)\s?([abcdsfxp\+]+)\s?([\u4e00-\u9fa5]+)?进度\s?([0-9]+)?\s?(.+)?$", raw_text)
    if not m:
        await UniMessage([At(flag="user", target=user_id), "命令解析失败，请检查输入格式"]).finish()
        return

    raw_level = m.group(1)
    raw_plan = m.group(2)

    logger.info(f"[{user_id}] 查询等级进度: {raw_level} {raw_plan}")

    logger.debug(f"[{user_id}] 1/3 获得用户鉴权凭证...")
    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)
    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(
        db_session, user_id, provider, use_personal_api=True
    )

    songs = list(MaiSongORM._cache.values())
    if not songs:
        await MaiSongORM.refresh_cache(db_session)
        songs = list(MaiSongORM._cache.values())

    logger.debug(f"[{user_id}] 2/3 发起 API 请求玩家所有成绩")
    scores = await score_provider.fetch_player_scoreslist(
        MaimaiPyParams(score_provider=provider, identifier=identifier)
    )
    try:
        data = get_level_process_data(songs, scores, raw_level, raw_plan)
    except ProcessDataError as e:
        await UniMessage([At(flag="user", target=user_id), str(e)]).finish()
        return

    logger.debug(f"[{user_id}] 3/3 渲染玩家数据...")
    img = ProcessPainter(data).draw_level()
    await UniMessage([At(flag="user", target=user_id), UniImage(raw=img.getvalue())]).finish()


@alconna_update.assign("songs")
@catch_exception()
async def handle_update_songs(
    event: Event,
    db_session: async_scoped_session,
):
    user_id = event.get_user_id()
    nb_config = get_driver().config

    if user_id not in nb_config.superusers:
        await UniMessage("更新乐曲信息需要管理员权限哦").finish()

    logger.info(f"[{user_id}] 更新乐曲信息数据库")

    from .updater.songs import update_song_database

    updated_count = await update_song_database(db_session)

    logger.info(f"[{user_id}] 乐曲信息数据库更新完成，共更新 {updated_count} 首乐曲")

    await UniMessage(
        [
            At(flag="user", target=user_id),
            f"乐曲信息数据库已更新完成，共更新 {updated_count} 首乐曲 ⭐",
        ]
    ).finish()


@alconna_update.assign("alias")
async def handle_update_aliases(
    event: Event,
    db_session: async_scoped_session,
):
    await handle_alias_update(event, db_session)


@alconna_update.assign("chart")
async def handle_update_chart(event: Event):
    from .updater.songs import update_local_chart_file

    await update_local_chart_file()

    await UniMessage(
        [
            At(flag="user", target=event.get_user_id()),
            "music_chart.json 文件已更新完成⭐",
        ]
    ).finish()


@alconna_fortune.handle()
async def handle_fortune(
    event: Event,
):
    user_id = event.get_user_id()

    logger.info(f"[{user_id}] 获取今日舞萌运势")

    fortune_message = await generate_today_fortune(user_id)

    await fortune_message.finish()


@alconna_analysis.handle()
@catch_exception()
async def handle_analysis(
    db_session: async_scoped_session,
    event: Event,
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()

    if not SONG_TAGS_DATA_AVAILABLE:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "管理员未配置乐曲标签，无法使用此功能喵",
            ]
        ).finish()
        return

    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)

    logger.info(f"[{user_id}] 获取玩家底力表, 查分器类型: {type(provider)}")
    logger.debug(f"[{user_id}] 1/4 获得用户鉴权凭证...")

    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(db_session, user_id, provider)
    params = score_provider.ParamsType(provider, identifier)

    logger.debug(f"[{user_id}] 2/4 发起 API 请求玩家所有成绩")
    if isinstance(provider, LXNSProvider):
        user_bind_info = await UserBindInfoORM.get_user_bind_info(db_session, user_id)
        if user_bind_info is None or user_bind_info.lxns_api_key is None:
            await UniMessage("你还没有绑定任何查分器喵，请先用 /bind 绑定一个查分器谢谢喵").finish()
            return  # Avoid TypeError.

        identifier.credentials = user_bind_info.lxns_api_key
        params = score_provider.ParamsType(provider, identifier)
    player_scores = await score_provider.fetch_player_scoreslist(params)

    logger.debug(f"[{user_id}] 3/4 计算玩家成分")
    player_scores.sort(key=lambda x: x.dx_rating, reverse=True)
    player_scores = player_scores[:100]
    player_strength = get_player_strength(player_scores)

    logger.debug(f"[{user_id}] 4/4 渲染玩家数据...")
    pic = draw_player_strength_analysis(player_strength)
    byte = image_to_bytes(pic)

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=byte)]).finish()


@alconna_trend.handle()
@catch_exception()
async def handle_trend(
    db_session: async_scoped_session,
    event: Event,
    score_provider: LXNSScoreProvider = Depends(get_lxns_provider),
):
    user_id = event.get_user_id()

    logger.info(f"[{user_id}] 获取玩家 Rating 趋势, 查分器类型: {type(score_provider)}")
    logger.debug(f"[{user_id}] 1/3 获得用户鉴权凭证...")

    user_bind_info = await UserBindInfoORM.get_user_bind_info(db_session, user_id)

    new_player_friend_code = None
    if user_bind_info is None:
        logger.warning(f"[{user_id}] 未能获取玩家码，数据库中不存在绑定的玩家数据")
        logger.debug(f"[{user_id}] 1/3 尝试通过 QQ 请求玩家数据")
        try:
            player_info = await score_provider.fetch_player_info_by_qq(user_id)
            new_player_friend_code = player_info.friend_code
        except ClientResponseError as e:
            logger.warning(f"[{user_id}] 无法通过 QQ 号请求玩家数据: {e.code}: {e.message}")

            await UniMessage(
                [
                    At(flag="user", target=user_id),
                    "查询 Rating 趋势的操作需要绑定落雪查分器喵，还请使用 /bind 指令进行绑定喵呜",
                ]
            ).finish()

            return

    friend_code = new_player_friend_code or user_bind_info.friend_code  # type: ignore
    if not friend_code:
        logger.warning(f"[{user_id}] 无法获取好友码，无法继续查询。")
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "无法获取好友码，请确认已绑定或查分器可用。",
            ]
        ).finish()
        return

    logger.debug(f"[{user_id}] 2/3 发起 API 请求玩家 Trend 趋势")

    trends = await score_provider.fetch_player_trend(friend_code)

    if len(trends) < 5:
        await UniMessage(
            [
                At(flag="user", target=user_id),
                "玩家 Rating 记录数据太少，无法进行渲染",
            ]
        ).finish()

    logger.debug(f"[{user_id}] 3/3 渲染玩家数据...")
    pic = draw_player_rating_trend(trends)
    byte = image_to_bytes(pic)

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=byte)]).finish()


@alconna_recommend.handle()
@catch_exception()
async def handle_recommend(
    db_session: async_scoped_session,
    event: Event,
    score_provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider),
):
    user_id = event.get_user_id()
    provider = await MaimaiPyScoreProvider.auto_get_score_provider(db_session, user_id)

    logger.info(f"[{user_id}] 获取玩家推分推荐, 查分器类型: {type(provider)}")
    logger.debug(f"[{user_id}] 1/4 获得用户鉴权凭证...")

    identifier = await MaimaiPyScoreProvider.auto_get_player_identifier(db_session, user_id, provider)
    params = score_provider.ParamsType(provider, identifier)

    logger.debug(f"[{user_id}] 2/4 发起 API 请求玩家所有成绩")
    if isinstance(provider, LXNSProvider):
        user_bind_info = await UserBindInfoORM.get_user_bind_info(db_session, user_id)
        if user_bind_info is None or user_bind_info.lxns_api_key is None:
            await UniMessage("你还没有绑定任何查分器喵，请先用 /bind 绑定一个查分器谢谢喵").finish()
            return  # Avoid TypeError.

        identifier.credentials = user_bind_info.lxns_api_key
        params = score_provider.ParamsType(provider, identifier)
    player_scores = await score_provider.fetch_player_scoreslist(params)

    logger.debug(f"[{user_id}] 3/4 计算推分推荐...")
    st = perf_counter()
    player_scores.sort(key=lambda x: x.dx_rating, reverse=True)
    min_dx_score = player_scores[:50][-1].dx_rating
    recommend_songs = get_player_raise_score_songs(player_scores, round(min_dx_score))
    et = perf_counter()
    logger.debug(f"[{user_id}] 3/4 计算推分推荐用时 {(et - st):.3f}s")

    logger.debug(f"[{user_id}] 4/4 渲染玩家数据...")
    pic = DrawScores().draw_rise(recommend_songs, round(min_dx_score))
    byte = image_to_bytes(pic)

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=byte)]).finish()


@alconna_maistatus.handle()
@catch_exception(reply_prefix="获取舞萌状态失败")
async def handle_maistatus(event: Event):
    user_id = event.get_user_id()

    logger.debug("正在获取舞萌服务器状态...")
    try:
        status_text = await get_maistatus()
    except Exception as e:
        status_text = f"服务器状态检测失败：{e}"

    await UniMessage([At(flag="user", target=user_id), status_text]).send()

    if not config.maistatus_url:
        return

    logger.debug("正在尝试获取舞萌状态截图...")
    try:
        st = perf_counter()
        png = await capture_maimai_status_png(config.maistatus_url)
        et = perf_counter()
        render_time_message = f"渲染用时 {et - st:.2f} 秒"
    except Exception as e:
        await UniMessage([At(flag="user", target=user_id), f"状态页截图渲染失败：{e}"]).finish()
        return

    await UniMessage([At(flag="user", target=user_id), UniImage(raw=png), render_time_message]).finish()


@alconna_rikka.handle()
async def handle_rikka(db_session: async_scoped_session, event: Event):
    from .utils import get_version

    version = get_version()
    total_songs_count = len(await MaiSongORM.get_all_song_ids(db_session))

    message = (
        "Rikka 插件信息\n"
        f"插件版本: {version}\n"
        f"Bot 乐曲数量: {total_songs_count}\n"
        f"机台源支持: {'已启用' if config.enable_arcade_provider else '未启用'}\n"
        f"标签数据: {'已启用' if SONG_TAGS_DATA_AVAILABLE else '未启用'}\n"
        f"状态页支持: {'已启用' if config.maistatus_url else '未启用'}\n"
    )

    await UniMessage(message).send()

    await handle_help(event)
